@@ -2,7 +2,7 @@
  *	HT Editor
  *	x86asm.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -84,7 +84,7 @@ x86addrcoding modrm32[3][8] = {
 {X86_REG_CX, X86_REG_NO, 0},
 {X86_REG_DX, X86_REG_NO, 0},
 {X86_REG_BX, X86_REG_NO, 0},
-{X86_REG_INVALID, X86_REG_INVALID, -1},                           /* special: SIB */
+{X86_REG_INVALID, X86_REG_INVALID, -1},		/* special: SIB */
 {X86_REG_NO, X86_REG_NO, 4},
 {X86_REG_SI, X86_REG_NO, 0},
 {X86_REG_DI, X86_REG_NO, 0}
@@ -95,7 +95,7 @@ x86addrcoding modrm32[3][8] = {
 {X86_REG_CX, X86_REG_NO, 1},
 {X86_REG_DX, X86_REG_NO, 1},
 {X86_REG_BX, X86_REG_NO, 1},
-{X86_REG_INVALID, X86_REG_INVALID, -1},                           /* special: SIB + disp8 */
+{X86_REG_INVALID, X86_REG_INVALID, -1},		/* special: SIB + disp8 */
 {X86_REG_BP, X86_REG_NO, 1},
 {X86_REG_SI, X86_REG_NO, 1},
 {X86_REG_DI, X86_REG_NO, 1}
@@ -106,7 +106,7 @@ x86addrcoding modrm32[3][8] = {
 {X86_REG_CX, X86_REG_NO, 4},
 {X86_REG_DX, X86_REG_NO, 4},
 {X86_REG_BX, X86_REG_NO, 4},
-{X86_REG_INVALID, X86_REG_INVALID, -1},                           /* special: SIB + disp32 */
+{X86_REG_INVALID, X86_REG_INVALID, -1},		/* special: SIB + disp32 */
 {X86_REG_BP, X86_REG_NO, 4},
 {X86_REG_SI, X86_REG_NO, 4},
 {X86_REG_DI, X86_REG_NO, 4}
@@ -190,7 +190,7 @@ int isnotwhitespace(char c)
  */
 
 x86asm::x86asm(int o, int a)
-: assembler(false)
+: Assembler(false)
 {
 	opsize=o;
 	addrsize=a;
@@ -256,11 +256,16 @@ void x86asm::emitimm(dword i, int size)
 	imm=i;
 }
 
-void x86asm::emitimm48(dword s, dword o)
+void x86asm::emitfarptr(dword s, dword o, bool big)
 {
-	immsize=6;
-	imm=o;
-	imm2=s;
+	if (big) {
+		immsize=6;
+		imm=o;
+		imm2=s;
+	} else {
+		immsize=4;
+		imm=(s<<16) | (o & 0xffff);
+	}
 }
 
 void x86asm::emitmodrm(int modrm)
@@ -307,7 +312,7 @@ void x86asm::emitsib_scale(int scale)
 
 asm_code *x86asm::encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address)
 {
-	assembler::encode(asm_insn, options, cur_address);
+	Assembler::encode(asm_insn, options, cur_address);
 	x86asm_insn *insn=(x86asm_insn*)asm_insn;
 	
 	addrsize_depend=0;
@@ -320,7 +325,7 @@ asm_code *x86asm::encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address)
 		
 	newcode();
 	namefound=0;
-	address=cur_address.addr32;
+	address=cur_address.addr32.offset;
 	esizes[0]=0;
 	esizes[1]=0;
 	esizes[2]=0;
@@ -455,7 +460,8 @@ int x86asm::encode_modrm(x86_insn_op *op, char size, int allow_reg, int allow_me
 		case X86_OPTYPE_MEM: {
 			if (!allow_mem) return 0;
 
-			int mindispsize=op->mem.disp ? simmsize(op->mem.disp) : 0;
+			// FIXME: !
+			int mindispsize=op->mem.disp ? simmsize(op->mem.disp, 4) : 0;
 			int addrsize=op->mem.addrsize;
 			if (addrsize==X86_ADDRSIZEUNKNOWN) addrsize=eaddrsize;
 			if (addrsize==X86_ADDRSIZE16) {
@@ -521,10 +527,11 @@ int x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eops
 			return 1;
 		case TYPE_A:
 /* direct address without ModR/M */
-			if ((op->type == X86_OPTYPE_FARPTR) && (op->size == 6)) {
-			    emitimm48(op->farptr.seg, op->farptr.offset);
+			if (op->type == X86_OPTYPE_FARPTR) {
+				int size = esizeop(xop->size, eopsize);
+				emitfarptr(op->farptr.seg, op->farptr.offset, size == 6);
 			} else {
-			    emitimm(op->imm, op->size);
+				emitimm(op->imm, op->size);
 			}
 			break;
 		case TYPE_C:
@@ -704,6 +711,8 @@ int x86asm::encode_sib_v(x86_insn_op *op, int mindispsize, int *_ss, int *_index
 int x86asm::esizeaddr(char c, int size)
 {
 	switch (c) {
+		case SIZE_A:
+			if (size==X86_ADDRSIZE16) return 4; else return 6;
 		case SIZE_B:
 			return 1;
 		case SIZE_W:
@@ -871,15 +880,16 @@ int x86asm::match_size(x86_insn_op *op, x86opc_insn_op *xop, int opsize)
 {
 	if ((op->type==X86_OPTYPE_EMPTY) && (xop->type==TYPE_0)) return 1;
 	if ((!op->size) && (xop->type!=TYPE_0)) return 1;
-	char *hsz=0;
+	char *hsz = NULL;
 	if (((op->type==X86_OPTYPE_MEM) && (op->mem.floatptr)) || (op->type==X86_OPTYPE_STX)) {
 		return (xop->size==flsz2hsz(op->size));
 	} else if (op->type==X86_OPTYPE_IMM) {
 		if (xop->type==TYPE_Is) {
-			hsz=immlsz2hsz(simmsize(op->imm), opsize);
+			hsz=immlsz2hsz(simmsize(op->imm, esizeop(xop->extendedsize, opsize)), opsize);
 		} else if (xop->type==TYPE_J) {
 			int size=esizeop(xop->size, opsize);
-			hsz=immlsz2hsz(simmsize(op->imm-address-code.size-size-1), opsize);
+			// FIXME: ?!
+			hsz=immlsz2hsz(simmsize(op->imm-address-code.size-size-1, 4), opsize);
 		} else {
 			hsz=immlsz2hsz(op->size, opsize);
 		}
@@ -909,7 +919,8 @@ int x86asm::match_allops(x86asm_insn *insn, x86opc_insn *xinsn, int opsize, int
 #define MATCHOPNAME_MATCH_IF_OPSIZE16	2
 #define MATCHOPNAME_MATCH_IF_OPSIZE32	3
 
-int x86asm::match_opcode_name(char *input_name, char *opcodelist_name) {
+int x86asm::match_opcode_name(char *input_name, char *opcodelist_name)
+{
 	if (opcodelist_name) {
 		if (strstr(opcodelist_name, "%c")) {
 			char n[32];
@@ -1071,7 +1082,7 @@ int x86asm::opfarptr(x86_insn_op *op, char *xop)
 	if (!fetch_number(&x, &offset)) return 0;
 	if (*x) return 0;
 	op->type=X86_OPTYPE_FARPTR;
-	op->size=6;
+	if (offset>0xffff) op->size=6; else op->size=4;
 	op->farptr.seg=seg;
 	op->farptr.offset=offset;
 	return 1;
@@ -1143,7 +1154,6 @@ int x86asm::opmem(x86asm_insn *asm_insn, x86_insn_op *op, char *s)
 		if (!strncmp(s, "ptr", 3)==0) return 0;
 		opsize=hsize;
 		s+=3;
-		if (!strchr(" \t", *s) || !*s) return 0;
 		while (strchr(" \t", *s) && *s) s++;
 	}
 
@@ -1169,7 +1179,6 @@ int x86asm::opmem(x86asm_insn *asm_insn, x86_insn_op *op, char *s)
 	dword disp=0;
 	int addrsize=X86_ADDRSIZEUNKNOWN;
 	int lasttokenreg=X86_REG_NO;
-	int lasttokendisp=0;
 
 	int sign=0;
 	char buf[64]; /* FIXME: possible buffer overflow */
@@ -1181,7 +1190,7 @@ cont:
 		if (strchr("*+-[]()", *s) && *s) {
 			s++;
 		} else {
-			while (!strchr(" \t*+-[]()", *s) || !*s) s++;
+			while (!strchr(" \t*+-[]()", *s) && *s) s++;
 		}
 		strncpy(buf, t, s-t);
 		buf[s-t]=0;
@@ -1248,16 +1257,15 @@ cont:
 		dword v;
 		
 		if (imm_eval_proc && imm_eval_proc(imm_eval_context, &t, &v)) {
-			lasttokendisp=disp;
 			if (sign) disp-=v; else disp+=v;
 			continue;
 		} else if (fetch_number(&t, &v)) {
-			lasttokendisp=disp;
 			if (sign) disp-=v; else disp+=v;
 			continue;
 		}
 		return 0;
 	}
+
 	if ((base==X86_REG_NO) && (index==X86_REG_NO)) {
 /* unsigned disp */
 		if (disp>0xffff) {
@@ -1326,7 +1334,7 @@ int x86asm::opspecialregs(x86_insn_op *op, char *xop)
 	return 0;
 }
 
-int x86asm::prepare_str(asm_insn *asm_insn, char *s)
+int x86asm::translate_str(asm_insn *asm_insn, const char *s)
 {
 	x86asm_insn *insn=(x86asm_insn*)asm_insn;
 	char *opp[3], op[3][256];
@@ -1334,11 +1342,31 @@ int x86asm::prepare_str(asm_insn *asm_insn, char *s)
 	opp[1]=op[1];
 	opp[2]=op[2];
 	for (int i=0; i<3; i++) insn->op[i].type=X86_OPTYPE_EMPTY;
-	splitstr(s, insn->n, (char**)&opp);
-	insn->name=insn->n;
+
 	insn->lockprefix=X86_PREFIX_NO;
 	insn->repprefix=X86_PREFIX_NO;
 	insn->segprefix=X86_PREFIX_NO;
+
+	const char *p = s, *a, *b;
+/* prefixes */
+	while (iswhitespace(*p)) p++;
+	a=p;
+	while (isnotwhitespace(*p)) p++;
+	b=p;
+	if ((strncmp(a, "rep", b-a) == 0) || (strncmp(a, "repe", b-a) == 0) ||
+	(strncmp(a, "repz", b-a) == 0)) {
+		insn->repprefix=X86_PREFIX_REPZ;
+		s = p;
+	} else if ((strncmp(a, "repne", b-a) == 0) || (strncmp(a, "repnz", b-a) == 0)) {
+		insn->repprefix=X86_PREFIX_REPNZ;
+		s = p;
+	} else if (strncmp(a, "lock", b-a) == 0) {
+		insn->lockprefix=X86_PREFIX_LOCK;
+		s = p;
+	}
+/**/
+	splitstr(s, insn->n, (char**)&opp);
+	insn->name=insn->n;
 	for (int i=0; i<3; i++) {
 		if (!*op[i]) break;
 
@@ -1356,17 +1384,31 @@ int x86asm::prepare_str(asm_insn *asm_insn, char *s)
 	return 1;
 }
 
-int x86asm::simmsize(dword imm)
+int x86asm::simmsize(dword imm, int immsize)
 {
-//	if (!imm) return 0;
-	if ((imm<=0x7f) || (imm>=0xffffff80)) return 1;
-	if ((imm<=0x7fff) || (imm>=0xffff8000)) return 2;
+	int i;
+	switch (immsize) {
+		case 1:
+			if (imm > 0xff) return 0;
+			i = (sint8)imm;
+			break;
+		case 2:
+			if (imm > 0xffff) return 0;
+			i = (sint16)imm;
+			break;
+		case 4:
+			if (imm > 0xffffffff) return 0;
+			i = (sint32)imm;
+			break;
+	}
+	if ((i >= -0x80) && (i < 0x80)) return 1;
+	if ((i >= -0x8000) && (i < 0x8000)) return 2;
 	return 4;
 }
 
-void x86asm::splitstr(char *s, char *name, char *op[3])
+void x86asm::splitstr(const char *s, char *name, char *op[3])
 {
-	char *a, *b;
+	const char *a, *b;
 	int wantbreak=0;
 	*name=0;
 	*op[0]=0;
