@@ -2,7 +2,7 @@
  *	HT Editor
  *	x86dis.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,6 +21,7 @@
 #include "stream.h"
 #include "store.h"
 #include "tools.h"
+#include "vxd.h"
 #include "x86dis.h"
 
 #include <stdio.h>
@@ -65,13 +66,14 @@ x86dis::~x86dis()
 {
 }
 
-dis_insn *x86dis::decode(byte *code, byte _maxlen, CPU_ADDR _addr)
+dis_insn *x86dis::decode(byte *code, byte Maxlen, CPU_ADDR Addr)
 {
 	ocodep=code;
 /* initialize */
 	codep=ocodep;
-	maxlen=_maxlen;
-	addr=_addr.addr32;
+	maxlen=Maxlen;
+	seg=Addr.addr32.seg;
+	addr=Addr.addr32.offset;
 	modrm=-1;
 	sib=-1;
 	memset(&insn, 0, sizeof(insn));
@@ -202,10 +204,12 @@ void x86dis::decode_insn(x86opc_insn *xinsn)
 			case SPECIAL_TYPE_PREFIX:
 				switch (c) {
 					case 0x0f:
-						insn.opcode=getbyte();
-						insn.opcodeclass=X86DIS_OPCODE_CLASS_EXT;
-						decode_insn(&x86_insns_ext[insn.opcode]);
-						break;
+						if (insn.opcodeclass == X86DIS_OPCODE_CLASS_STD) {
+							insn.opcode = getbyte();
+							insn.opcodeclass = X86DIS_OPCODE_CLASS_EXT;
+							decode_insn(&x86_insns_ext[insn.opcode]);
+							break;
+						}							    
 					default:
 						invalidate();
 						break;
@@ -249,7 +253,7 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 		case TYPE_A: {
 /* direct address without ModR/M */
 			op->type=X86_OPTYPE_FARPTR;
-			op->size=esizeaddr(xop->size);
+			op->size=esizeop(xop->size);
 			switch (op->size) {
 				case 4:
 					op->farptr.offset=getword();
@@ -487,6 +491,13 @@ void x86dis::decode_sib(x86_insn_op *op, int mod)
 	}
 }
 
+dis_insn *x86dis::duplicateInsn(dis_insn *disasm_insn)
+{
+	x86dis_insn *insn = (x86dis_insn *)malloc(sizeof (x86dis_insn));
+	*insn = *(x86dis_insn *)disasm_insn;
+	return insn;
+}
+
 int x86dis::esizeaddr(char c)
 {
 	switch (c) {
@@ -580,10 +591,13 @@ dword x86dis::getdword()
 	}
 }
 
-int x86dis::getmaxopcodelength()
+void x86dis::getOpcodeMetrics(int &min_length, int &max_length, int &min_look_ahead, int &avg_look_ahead, int &addr_align)
 {
-	/* x86 opcodes are <= 15 bytes */
-	return 15;
+	min_length = 1;
+	max_length = 15;
+	min_look_ahead = 120;    // 1/2/3/4/5/6/8/10/12/15
+	avg_look_ahead = 24;     // 1/2/3/4/6/8/12/24
+	addr_align = 1;
 }
 
 int x86dis::getmodrm()
@@ -592,44 +606,22 @@ int x86dis::getmodrm()
 	return modrm;
 }
 
+char *x86dis::getName()
+{
+	return "x86/Disassembler";
+}
+
 int x86dis::getsib()
 {
 	if (sib==-1) sib=getbyte();
 	return sib;
 }
 
-byte x86dis::getsize(dis_insn *disasm_insn)
+byte x86dis::getSize(dis_insn *disasm_insn)
 {
 	return ((x86dis_insn*)disasm_insn)->size;
 }
 
-void x86dis::hexd(char **s, int size, int options, int imm)
-{
-	char ff[16];
-	char *f=(char*)&ff;
-	char *t=*s;
-	f+=sprintf(f, "%%");
-	if (options & X86DIS_STYLE_SIGNED) {
-		if (!(options & X86DIS_STYLE_HEX_NOZEROPAD)) f+=sprintf(f, "0%d", size);
-		*(f++)='d';
-		*f=0;
-		(*s)+=sprintf(*s, ff, imm);
-	} else {
-		if (options & X86DIS_STYLE_HEX_CSTYLE) *(f++)='#';
-		if (!(options & X86DIS_STYLE_HEX_NOZEROPAD)) f+=sprintf(f, "0%d", size);
-		if (options & X86DIS_STYLE_HEX_UPPERCASE) *(f++)='X'; else
-			*(f++)='x';
-		if (options & X86DIS_STYLE_HEX_ASMSTYLE) *(f++)='h';
-		*f=0;
-		(*s)+=sprintf(*s, ff, imm);
-		if ((options & X86DIS_STYLE_HEX_NOZEROPAD) && (*t-'0'>9)) {
-			memmove(t+1, t, strlen(t)+1);
-			*t='0';
-			(*s)++;
-		}
-	}
-}
-
 void x86dis::invalidate()
 {
 	insn.invalid=1;
@@ -653,7 +645,7 @@ int x86dis::load(ht_object_stream *f)
 	return f->get_error();
 }
 
-OBJECT_ID x86dis::object_id()
+OBJECT_ID x86dis::object_id() const
 {
 	return ATOM_DISASM_X86;
 }
@@ -708,11 +700,12 @@ void x86dis::prefixes()
 
 int x86dis::special_param_ambiguity(x86dis_insn *disasm_insn)
 {
-	int regc=0, memc=0;
+	int regc=0, memc=0, segc=0;
 	for (int i=0; i<3; i++) {
 		switch (disasm_insn->op[i].type) {
-			case X86_OPTYPE_REG:
 			case X86_OPTYPE_SEG:
+				segc++;
+			case X86_OPTYPE_REG:
 			case X86_OPTYPE_CRX:
 			case X86_OPTYPE_DRX:
 			case X86_OPTYPE_TRX:
@@ -725,24 +718,38 @@ int x86dis::special_param_ambiguity(x86dis_insn *disasm_insn)
 				break;
 		}
 	}
-	return (memc && (!regc));
+	return (memc && !regc)
+		|| (memc && segc)
+		|| (strcmp(disasm_insn->name, "movzx") == 0)
+		|| (strcmp(disasm_insn->name, "movsx") == 0);
 }
 
 void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *op, bool explicit_params)
 {
+	const char *cs_default = get_cs(e_cs_default);
+	const char *cs_number = get_cs(e_cs_number);
+	const char *cs_symbol = get_cs(e_cs_symbol);
+	
 	*opstrlen=0;
 	switch (op->type) {
 		case X86_OPTYPE_IMM: {
 			CPU_ADDR a;
-			a.addr32=op->imm;
+			// FIXME: hack
+			if ((insn->name[0]=='j') || strcmp(insn->name, "call")==0) {
+				a.addr32.seg = seg;
+			} else {
+				a.addr32.seg = 0;
+			}
+			a.addr32.offset = op->imm;
 			int slen;
-			char *s=(addr_sym_func) ? addr_sym_func(a, &slen) : 0;
+			char *s=(addr_sym_func) ? addr_sym_func(a, &slen, addr_sym_func_context) : NULL;
 			if (s) {
 				memmove(opstr, s, slen);
 				opstr[slen]=0;
 				*opstrlen=slen;
 			} else {
 				char *g=opstr;
+				strcpy(g, cs_number); g += strlen(cs_number);
 				switch (op->size) {
 					case 1:
 						hexd(&g, 2, options, op->imm);
@@ -754,6 +761,7 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 						hexd(&g, 8, options, op->imm);
 						break;
 				}
+				strcpy(g, cs_default); g += strlen(cs_default);
 			}
 			break;
 		}
@@ -830,13 +838,15 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 				}
 			}
 			if (insn->eaddrsize==X86_ADDRSIZE16) reg=1; else reg=2;
-			if (insn->segprefix!=X86_PREFIX_NO) d+=sprintf(d, "%s:", x86_segs[insn->segprefix]);
+			if (insn->segprefix!=X86_PREFIX_NO) d+=sprintf(d, "%s%s:%s", x86_segs[insn->segprefix], cs_symbol, cs_default);
+			strcpy(d, cs_symbol); d += strlen(cs_symbol);
 			*(d++)='[';
+			strcpy(d, cs_default); d += strlen(cs_default);
 
 			int optimize_addr=options & X86DIS_STYLE_OPTIMIZE_ADDR;
 			int first=1;
 			if (optimize_addr && (op->mem.base!=X86_REG_NO) && (op->mem.base==op->mem.index)) {
-				d+=sprintf(d, "%s*%d", x86_regs[reg][op->mem.index], op->mem.scale+1);
+				d+=sprintf(d, "%s%s*%s%d%s", x86_regs[reg][op->mem.index], cs_symbol, cs_number, op->mem.scale+1, cs_default);
 				first=0;
 			} else {
 				if (op->mem.base!=X86_REG_NO) {
@@ -848,18 +858,23 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 					if ((op->mem.scale==1)) {
 						d+=sprintf(d, "%s", x86_regs[reg][op->mem.index]);
 					} else {
-						d+=sprintf(d, "%s*%d", x86_regs[reg][op->mem.index], op->mem.scale);
+						d+=sprintf(d, "%s%s*%s%d%s", x86_regs[reg][op->mem.index], cs_symbol, cs_number, op->mem.scale, cs_default);
 					}
 					first=0;
 				}
 			}
 			if ((!optimize_addr && op->mem.hasdisp) || (optimize_addr && op->mem.disp) || first) {
 				CPU_ADDR a;
-				a.addr32=op->mem.disp;
+				a.addr32.seg = 0; // FIXME: not ok
+				a.addr32.offset=op->mem.disp;
 				int slen;
-				char *s=(addr_sym_func) ? addr_sym_func(a, &slen) : 0;
+				char *s=(addr_sym_func) ? addr_sym_func(a, &slen, addr_sym_func_context) : 0;
 				if (s) {
-					if (!first) *(d++)='+';
+					if (!first) {
+						strcpy(d, cs_symbol); d += strlen(cs_symbol);
+						*(d++)='+';
+						strcpy(d, cs_default); d += strlen(cs_default);
+					}
 					memmove(d, s, slen);
 					d+=slen;
 					*opstrlen=d-opstr;
@@ -869,44 +884,67 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 						case X86_ADDRSIZE16:
 							q=op->mem.disp;
 							if (!first) {
+								strcpy(d, cs_symbol); d += strlen(cs_symbol);
 								if (op->mem.disp&0x80000000) {
 									*(d++)='-';
 									q=-q;
 								} else *(d++)='+';
 							}
+							strcpy(d, cs_number); d += strlen(cs_number);
 							hexd(&d, 4, options, q);
+							strcpy(d, cs_default); d += strlen(cs_default);
 							break;
 						case X86_ADDRSIZE32:
 							q=op->mem.disp;
 							if (!first) {
+								strcpy(d, cs_symbol); d += strlen(cs_symbol);
 								if (op->mem.disp&0x80000000) {
 									*(d++)='-';
 									q=-q;
 								} else *(d++)='+';
 							}
+							strcpy(d, cs_number); d += strlen(cs_number);
 							hexd(&d, 8, options, q);
+							strcpy(d, cs_default); d += strlen(cs_default);
 							break;
 					}
 				}
 			}
-			if (*opstrlen) (*opstrlen)++;
+			strcpy(d, cs_symbol); d += strlen(cs_symbol);
 			*(d++)=']';
+			strcpy(d, cs_default); d += strlen(cs_default);
+			if (*opstrlen)
+				*opstrlen += strlen(cs_symbol) + 1 + strlen(cs_default);
 			*d=0;
 			break;
 		}
 		case X86_OPTYPE_FARPTR: {
-			char *g=opstr;
-			switch (op->size) {
-				case 4:
-					hexd(&g, 4, options, op->farptr.seg);
-					*(g++)=':';
-					hexd(&g, 4, options, op->farptr.offset);
-					break;
-				case 6:
-					hexd(&g, 4, options, op->farptr.seg);
-					*(g++)=':';
-					hexd(&g, 8, options, op->farptr.offset);
-					break;
+			CPU_ADDR a;
+/*			a.addr32.seg = op->farptr.seg;
+			a.addr32.offset = op->farptr.offset;
+*/
+			a.addr32.seg = op->farptr.seg;
+			a.addr32.offset = op->farptr.offset;
+			int slen;
+			char *s=(addr_sym_func) ? addr_sym_func(a, &slen, addr_sym_func_context) : 0;
+			if (s) {
+				memmove(opstr, s, slen);
+				opstr[slen]=0;
+				*opstrlen=slen;
+			} else {
+				char *g=opstr;
+				hexd(&g, 4, options, op->farptr.seg);
+				strcpy(g, cs_symbol); g += strlen(cs_symbol);
+				*(g++)=':';
+				strcpy(g, cs_default); g += strlen(cs_default);
+				switch (op->size) {
+					case 4:
+						hexd(&g, 4, options, op->farptr.offset);
+						break;
+					case 6:
+						hexd(&g, 8, options, op->farptr.offset);
+						break;
+				}
 			}
 			break;
 		}
@@ -917,13 +955,16 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 
 void x86dis::str_format(char **str, char **format, char *p, char *n, char *op[3], int oplen[3], char stopchar, int print)
 {
+	const char *cs_default = get_cs(e_cs_default);
+	const char *cs_symbol = get_cs(e_cs_symbol);
+
 	char *f=*format;
 	char *s=*str;
 	while (*f) {
 		if (*f==stopchar) break;
 		switch (*f) {
 			case '\t':
-				if (print) do *(s++)=' '; while ((s-insnstr) % X86DIS_STYLE_TABSIZE);
+				if (print) do *(s++)=' '; while ((s-insnstr) % DIS_STYLE_TABSIZE);
 				break;
 			case DISASM_STRF_VAR:
 				f++;
@@ -954,7 +995,10 @@ void x86dis::str_format(char **str, char **format, char *p, char *n, char *op[3]
 						memmove(s, t, tl);
 						s+=tl;
 						*s=0;
-					} else s+=sprintf(s, t);
+					} else {
+						strcpy(s, t);
+						s += strlen(s);
+					}
 				}
 				break;
 			case DISASM_STRF_COND: {
@@ -986,7 +1030,12 @@ void x86dis::str_format(char **str, char **format, char *p, char *n, char *op[3]
 				break;
 			}
 			default:
-				if (print) *(s++)=*f;
+				if (print) {
+					bool x = (strchr(",.-=+-*/[]()", *f) != NULL) && *f;
+					if (x) { strcpy(s, cs_symbol); s += strlen(cs_symbol); }
+					*(s++) = *f;
+					if (x) { strcpy(s, cs_default); s += strlen(cs_default); }
+				}
 		}
 		f++;
 	}
@@ -1000,31 +1049,32 @@ char *x86dis::str(dis_insn *disasm_insn, int options)
 	return strf(disasm_insn, options, DISASM_STRF_DEFAULT_FORMAT);
 }
 
-char *x86dis::strf(dis_insn *disasm_insn, int _options, char *format)
+char *x86dis::strf(dis_insn *disasm_insn, int opt, char *format)
 {
-	x86dis_insn *insn=(x86dis_insn*)disasm_insn;
+	x86dis_insn *insn = (x86dis_insn*)disasm_insn;
 	char prefix[64];
-	char *p=prefix;
-	options=_options;
-	*p=0;
-	if (insn->lockprefix==X86_PREFIX_LOCK) p+=sprintf(p, "lock ");
-	if (insn->repprefix==X86_PREFIX_REPZ) {
-		p+=sprintf(p, "repz ");
-	} else if (insn->repprefix==X86_PREFIX_REPNZ) {
-		p+=sprintf(p, "repnz ");
+	char *p = prefix;
+	options = opt;
+	*p = 0;
+	if (insn->lockprefix == X86_PREFIX_LOCK) p+=sprintf(p, "lock ");
+	if (insn->repprefix == X86_PREFIX_REPZ) {
+		p += sprintf(p, "repz ");
+	} else if (insn->repprefix == X86_PREFIX_REPNZ) {
+		p += sprintf(p, "repnz ");
 	}
-	if ((p!=prefix) && (*(p-1)==' ')) {
+	if ((p != prefix) && (*(p-1) == ' ')) {
 		p--;
-		*p=0;
+		*p = 0;
 	}
 
-	bool explicit_params=((options & X86DIS_STYLE_EXPLICIT_MEMSIZE) ||
+	bool explicit_params = ((options & X86DIS_STYLE_EXPLICIT_MEMSIZE) ||
 		special_param_ambiguity(insn));
 
 	char ops[3][512];	/* FIXME: possible buffer overflow ! */
 	char *op[3];
 	int oplen[3];
 
+	if (options & DIS_STYLE_HIGHLIGHT) enable_highlighting();
 	for (int i=0; i<3; i++) {
 		op[i]=(char*)&ops[i];
 		str_op(op[i], &oplen[i], insn, &insn->op[i], explicit_params);
@@ -1043,6 +1093,7 @@ char *x86dis::strf(dis_insn *disasm_insn, int _options, char *format)
 			break;
 	}
 	str_format(&s, &format, prefix, n, op, oplen, 0, 1);
+	disable_highlighting();
 	return insnstr;
 }
 
@@ -1052,8 +1103,81 @@ void x86dis::store(ht_object_stream *f)
 	PUT_INT_HEX(f, addrsize);
 }
 
-bool x86dis::valid_insn(dis_insn *disasm_insn)
+bool x86dis::validInsn(dis_insn *disasm_insn)
 {
 	return !((x86dis_insn *)disasm_insn)->invalid;
 }
 
+/*
+ *	CLASS x86dis_vxd
+ */
+
+x86dis_vxd::x86dis_vxd()
+{
+}
+
+x86dis_vxd::x86dis_vxd(int opsize, int addrsize)
+: x86dis(opsize, addrsize)
+{
+}
+
+x86dis_vxd::~x86dis_vxd()
+{
+}
+
+dis_insn *x86dis_vxd::decode(byte *code, byte maxlen, CPU_ADDR addr)
+{
+	if ((maxlen >= 6) && (code[0] == 0xcd) && (code[1] == 0x20)) {
+		insn.name = "VxDCall";
+		insn.size = 6;
+		vxd_t *v = find_vxd(vxds, *(word*)(code+4));
+
+		if (v) {
+			insn.op[0].type = X86_OPTYPE_USER;
+			insn.op[0].user[0].i = *(word*)(code+4);
+			insn.op[0].user[1].p = v->name;
+			char *vs = NULL;
+			if (v->services) vs = find_vxd_service(v->services, *(word*)(code+2) & 0x7fff);
+			if (vs) {
+				insn.op[1].type = X86_OPTYPE_USER;
+				insn.op[1].user[0].i = *(word*)(code+2);
+				insn.op[1].user[1].p = vs;
+			} else {
+				insn.op[1].type = X86_OPTYPE_IMM;
+				insn.op[1].size = 2;
+				insn.op[1].imm = *(word*)(code+2);
+			}
+		} else {
+			insn.op[0].type = X86_OPTYPE_IMM;
+			insn.op[0].size = 2;
+			insn.op[0].imm = *(word*)(code+4);
+
+			insn.op[1].type = X86_OPTYPE_IMM;
+			insn.op[1].size = 2;
+			insn.op[1].imm = *(word*)(code+2);
+		}
+		insn.op[2].type = X86_OPTYPE_EMPTY;
+		insn.lockprefix = X86_PREFIX_NO;
+		insn.repprefix = X86_PREFIX_NO;
+		insn.segprefix = X86_PREFIX_NO;
+		return &insn;
+	}
+	return x86dis::decode(code, maxlen, addr);
+}
+
+OBJECT_ID x86dis_vxd::object_id() const
+{
+	return ATOM_DISASM_X86_VXD;
+}
+
+void x86dis_vxd::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *op, bool explicit_params)
+{
+	if (op->type == X86_OPTYPE_USER) {
+		*opstrlen = 0;
+		strcpy(opstr, (char*)op->user[1].p);
+	} else {
+		x86dis::str_op(opstr, opstrlen, insn, op, explicit_params);
+	}
+}
+
+
