@@ -2,7 +2,7 @@
  *	HT Editor
  *	x86opc.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -72,7 +72,6 @@
 #define Ew	TYPE_E, 0, SIZE_W, SIZE_W
 #define Ed	TYPE_E, 0, SIZE_D, SIZE_D
 #define Eq	TYPE_E, 0, SIZE_Q, SIZE_Q
-#define Ep	TYPE_E, 0, SIZE_P, SIZE_P
 #define Ev	TYPE_E, 0, SIZE_V, SIZE_V
 #define Es	TYPE_E, 0, SIZE_S, SIZE_S
 #define El	TYPE_E, 0, SIZE_L, SIZE_L
@@ -688,10 +687,10 @@ x86opc_insn x86_insns_ext[256] = {
 /* B0 */
 {"cmpxchg", {{Eb}, {Gb}}},
 {"cmpxchg", {{Ev}, {Gv}}},
-{"lss", {{M}}},
+{"lss", {{Gv}, {Mp}}},
 {"btr", {{Ev}, {Gv}}},
-{"lfs", {{M}}},
-{"lgs", {{M}}},
+{"lfs", {{Gv}, {Mp}}},
+{"lgs", {{Gv}, {Mp}}},
 {"movzx", {{Gv}, {Eb}}},
 {"movzx", {{Gv}, {Ew}}},
 /* B8 */
@@ -916,9 +915,9 @@ x86opc_insn x86_group_insns[X86_GROUPS][8] = {
 {"inc", {{Ev}}},
 {"dec", {{Ev}}},
 {"call", {{Ev}}},
-{"call", {{Ep}}},
+{"call", {{Mp}}},
 {"jmp", {{Ev}}},
-{"jmp", {{Ep}}},
+{"jmp", {{Mp}}},
 {"push", {{Ev}}},
 {0}
 },
