@@ -1,8 +1,8 @@
 /* 
- *	The HT Editor
+ *	HT Editor
  *	x86opc.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -60,6 +60,11 @@
 // user defined types start here
 #define X86_OPTYPE_USER		32
 
+union int_or_ptr {
+	int i;
+	void *p;
+};
+
 struct x86_insn_op {
 	int type;
 	int size;
@@ -85,12 +90,13 @@ struct x86_insn_op {
 		int trx;
 		int stx;
 		int mmx;
-		int user[4];
+		int_or_ptr user[4];
 	};
 };
 
 #define TYPE_0			0
-#define TYPE_A			1		/* direct address without ModR/M */
+#define TYPE_A			1		/* direct address without ModR/M (generally
+							   like imm, but can be 16:32 = 48 bit) */
 #define TYPE_C			2		/* reg of ModR/M picks control register */
 #define TYPE_D			3		/* reg of ModR/M picks debug register */
 #define TYPE_E			4		/* ModR/M (general reg or memory) */
