@@ -2,7 +2,7 @@
  *	HT Editor
  *	cstream.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,23 +21,25 @@
 #define __CSTREAM_H__
 
 #include "global.h"
-//#include "minilzo.h"
 #include "stream.h"
 
 /*
- *	NEVER use ht_compressed_stream for both reading and writing!!
+ *	NEVER use ht_compressed_stream for both reading and writing!
  */
+ 
+#define COMPRESSED_STREAM_DEFAULT_GRANULARITY 10240
+
 class ht_compressed_stream: public ht_layer_stream {
 protected:
 	byte *buffer;
 	UINT buffersize;
 	UINT bufferpos;
 public:
-			void	init(ht_stream *stream, bool own_stream, UINT granularity=10240);
+			void	init(ht_stream *stream, bool own_stream, UINT granularity=COMPRESSED_STREAM_DEFAULT_GRANULARITY);
 	virtual	void	done();
 /* overwritten */
 	virtual	UINT	read(void *buf, UINT size);
-	virtual	UINT	write(void *buf, UINT size);
+	virtual	UINT	write(const void *buf, UINT size);
 protected:
 			bool flush_compressed();
 			bool flush_uncompressed();
