@@ -2,7 +2,7 @@
  *	HT Editor
  *	elfstruc.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -76,9 +76,19 @@ typedef word elf64_quarter;
 
 #define ELFOSABI_SYSV		0
 #define ELFOSABI_HPUX		1
+#define ELFOSABI_NETBSD		2
 #define ELFOSABI_LINUX		3
-#define ELFOSABI_STANDALONE	255
+#define ELFOSABI_HURD		4
+#define ELFOSABI_86OPEN		5
+#define ELFOSABI_SOLARIS		6
+#define ELFOSABI_MONTEREY	7
+#define ELFOSABI_IRIX		8
+#define ELFOSABI_FREEBSD		9
+#define ELFOSABI_TRU64		10
+#define ELFOSABI_MODESTO		11
+#define ELFOSABI_OPENBSD		12
 #define ELFOSABI_ARM		97
+#define ELFOSABI_STANDALONE	255
 
 /* e_type */
 #define ELF_ET_NONE			0
@@ -200,12 +210,12 @@ struct ELF_HEADER64 {
 
 /* indices */
 #define ELF_SHN_UNDEF		0
-#define ELF_SHN_ABS			0xfff1
+#define ELF_SHN_ABS		0xfff1
 #define ELF_SHN_COMMON		0xfff2
 
 /* sh_type */
 #define ELF_SHT_NULL		0
-#define ELF_SHT_PROGBITS		1
+#define ELF_SHT_PROGBITS	1
 #define ELF_SHT_SYMTAB		2
 #define ELF_SHT_STRTAB		3
 #define ELF_SHT_RELA		4
@@ -213,7 +223,7 @@ struct ELF_HEADER64 {
 #define ELF_SHT_DYNAMIC		6
 #define ELF_SHT_NOTE		7
 #define ELF_SHT_NOBITS		8
-#define ELF_SHT_REL			9
+#define ELF_SHT_REL		9
 #define ELF_SHT_SHLIB		10
 #define ELF_SHT_DYNSYM		11
 
@@ -222,14 +232,15 @@ struct ELF_HEADER64 {
 #define ELF_SHT_PREINIT_ARRAY	16
 
 /* sh_flags */
-#define ELF_SHF_WRITE		1
-#define ELF_SHF_ALLOC		2
-#define ELF_SHF_EXECINSTR	4
-#define ELF_SHF_MERGE		8
-#define ELF_SHF_STRINGS		16
-#define ELF_SHF_INFO_LINK	32
-#define ELF_SHF_LINK_ORDER	64
-#define ELF_SHF_OS_NONCONFORMING	128
+#define ELF_SHF_WRITE			(1<<0)
+#define ELF_SHF_ALLOC			(1<<1)
+#define ELF_SHF_EXECINSTR		(1<<2)
+// 1<<3 missing
+#define ELF_SHF_MERGE			(1<<4)
+#define ELF_SHF_STRINGS			(1<<5)
+#define ELF_SHF_INFO_LINK		(1<<6)
+#define ELF_SHF_LINK_ORDER		(1<<7)
+#define ELF_SHF_OS_NONCONFORMING	(1<<8)
 
 struct ELF_SECTION_HEADER32 {
 	elf32_word sh_name;
@@ -310,6 +321,10 @@ struct ELF_PROGRAM_HEADER64 {
 #define ELF32_ST_TYPE(i)		((i)&0xf)
 #define ELF32_ST_INFO(b,t)	(((b)>>4)|((t)&0xf))
 
+#define ELF64_ST_BIND(i)		((i)>>4)
+#define ELF64_ST_TYPE(i)		((i)&0xf)
+#define ELF64_ST_INFO(b,t)	(((b)>>4)|((t)&0xf))
+
 struct ELF_SYMBOL32 {
 	elf32_word st_name;
 	elf32_addr st_value;
@@ -384,4 +399,11 @@ extern byte ELF_SYMBOL64_struct[];
 extern byte ELF_REL64_struct[];
 extern byte ELF_RELA64_struct[];
 
+struct ELFAddress {
+	union {
+		dword a32;
+		qword a64;
+	};
+};
+
 #endif /* __ELFSTRUC_H__ */
