@@ -1,8 +1,8 @@
 /* 
  *	HT Editor
- *	viewers.cc
+ *	formats.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -25,6 +25,7 @@
 #include "class.h"
 #include "hthex.h"
 #include "httext.h"
+#include "htdisasm.h"
 #include "htfinfo.h"
 #include "htelf.h"
 #include "htcoff.h"
@@ -38,6 +39,7 @@ format_viewer_if *format_viewer_ifs[] =
 {
 	&hthex_if,
 	&httext_if,
+	&htdisasm_if,
 	&htfinfo_if,
 	&htelf_if,
 	&htmz_if,
