@@ -2,7 +2,7 @@
  *	HT Editor
  *	htatom.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -26,8 +26,8 @@
 typedef unsigned int HT_ATOM;
 
 void *find_atom(HT_ATOM atom);
-HT_ATOM find_atom_rev(void *data);
-bool register_atom(HT_ATOM atom, void *data);
+HT_ATOM find_atom_rev(const void *data);
+bool register_atom(HT_ATOM atom, const void *data);
 bool unregister_atom(HT_ATOM atom);
 
 /*
