@@ -2,7 +2,7 @@
  *	HT Editor
  *	htctrl.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -18,11 +18,17 @@
  *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "htobj.h"
+#include "htctrl.h"
+
+static char sglobalerror[GLOBAL_ERROR_SIZE];
+char *globalerror = sglobalerror;
 
 ht_view *app;
 ht_view *baseview;
 screendrawbuf *screen;
-char *this_app;
+ht_list *virtual_fs_list;
+void *project;
+char appname[HT_NAME_MAX+1];
 int some_analyser_active = 0;
 int num_ops_parsed = 0;
+
