@@ -2,7 +2,7 @@
  *	HT Editor
  *	htctrl.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,12 +21,18 @@
 #ifndef __HTCTRL_H__
 #define __HTCTRL_H__
 
+#include "htdata.h"
 #include "htobj.h"
 
+#define GLOBAL_ERROR_SIZE 1024
+extern char *globalerror;
+
 extern ht_view *app;
 extern ht_view *baseview;
 extern screendrawbuf *screen;
-extern char *this_app;
+extern ht_list *virtual_fs_list;
+extern void *project;
+extern char appname[HT_NAME_MAX+1];
 extern int some_analyser_active;
 extern int num_ops_parsed;
 
