@@ -2,7 +2,7 @@
  *	HT Editor
  *	htdata.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,11 +21,30 @@
 #include "htatom.h"
 #include "htdata.h"
 #include "htdebug.h"
+#include "stream.h"
 #include "tools.h"
 
 #include <string.h>
 #include <stdlib.h>
 
+#define NEW_TREE_ENUM
+
+int (*qsort_compare_compare_keys)(ht_data *key_a, ht_data *key_b);
+
+int qsort_compare(const void *e1, const void *e2)
+{
+	ht_data *d1=*(ht_data **)e1;
+	ht_data *d2=*(ht_data **)e2;
+	return qsort_compare_compare_keys(d1, d2);
+}
+
+int qsort_compare_keys_tree_node(const void *e1, const void *e2)
+{
+	ht_tree_node *d1=*(ht_tree_node **)e1;
+	ht_tree_node *d2=*(ht_tree_node **)e2;
+	return qsort_compare_compare_keys(d1->key, d2->key);
+}
+
 /*
  *	CLASS ht_data_uint
  */
@@ -37,16 +56,16 @@ ht_data_uint::ht_data_uint(UINT v)
 
 int ht_data_uint::load(ht_object_stream *s)
 {
-	value=s->get_int_hex(4, 0);
+	value=s->getIntHex(4, NULL);
 	return s->get_error();
 }
 
 void ht_data_uint::store(ht_object_stream *s)
 {
-	s->put_int_hex(value, 4, 0);
+	s->putIntHex(value, 4, NULL);
 }
 
-OBJECT_ID ht_data_uint::object_id()
+OBJECT_ID ht_data_uint::object_id() const
 {
 	return ATOM_HT_DATA_UINT;
 }
@@ -62,16 +81,16 @@ ht_data_dword::ht_data_dword(dword v)
 
 int ht_data_dword::load(ht_object_stream *s)
 {
-	value=s->get_int_hex(4, 0);
+	value=s->getIntHex(4, NULL);
 	return s->get_error();
 }
 
 void ht_data_dword::store(ht_object_stream *s)
 {
-	s->put_int_hex(value, 4, 0);
+	s->putIntHex(value, 4, NULL);
 }
 
-OBJECT_ID ht_data_dword::object_id()
+OBJECT_ID ht_data_dword::object_id() const
 {
 	return ATOM_HT_DATA_DWORD;
 }
@@ -80,7 +99,7 @@ OBJECT_ID ht_data_dword::object_id()
  *	CLASS ht_data_ptr
  */
 
-ht_data_ptr::ht_data_ptr(void *v)
+ht_data_ptr::ht_data_ptr(const void *v)
 {
 	value=v;
 }
@@ -89,14 +108,14 @@ ht_data_ptr::ht_data_ptr(void *v)
  *	CLASS ht_data_mem
  */
 
-ht_data_mem::ht_data_mem(void *v, UINT _size)
+ht_data_mem::ht_data_mem(const void *v, UINT _size)
 {
 	size=_size;
 	if (size) {
 		value=malloc(size);
 		memmove(value, v, size);
 	} else {
-		value=0;
+		value=NULL;
 	}
 }
 
@@ -107,16 +126,16 @@ ht_data_mem::~ht_data_mem()
 
 int ht_data_mem::load(ht_object_stream *s)
 {
-	value=s->get_binary(size, 0);
+	value=s->getBinary(size, NULL);
 	return s->get_error();
 }
 
 void ht_data_mem::store(ht_object_stream *s)
 {
-	s->put_binary(value, size, 0);
+	s->putBinary(value, size, NULL);
 }
 
-OBJECT_ID ht_data_mem::object_id()
+OBJECT_ID ht_data_mem::object_id() const
 {
 	return ATOM_HT_DATA_MEM;
 }
@@ -168,7 +187,12 @@ ht_data *ht_tree::enum_prev(ht_data **value, ht_data *nextkey)
 
 ht_data *ht_tree::get(ht_data *key)
 {
-	return 0;
+	return NULL;
+}
+
+ht_data *ht_tree::get_insert(ht_data *key)
+{
+	return NULL;
 }
 
 bool ht_tree::insert(ht_data *key, ht_data *data)
@@ -176,6 +200,10 @@ bool ht_tree::insert(ht_data *key, ht_data *data)
 	return false;
 }
 
+void ht_tree::set_compare_keys(compare_keys_func_ptr new_compare_keys)
+{
+}
+
 /*
  *	CLASS ht_stree	(simple tree)
  */
@@ -183,7 +211,7 @@ bool ht_tree::insert(ht_data *key, ht_data *data)
 void ht_stree::init(compare_keys_func_ptr compare_keys)
 {
 	ht_tree::init(compare_keys);
-	root=0;
+	root=NULL;
 	node_count=0;
 }
 
@@ -194,36 +222,34 @@ void ht_stree::done()
 
 void ht_stree::destroy()
 {
-	if (root) {
-		free_all(root);
-		root=0;
+	empty();
+	done();
+}
+
+void ht_stree::set_compare_keys(compare_keys_func_ptr new_compare_keys)
+{
+	if (node_count > 1) {
+		ht_tree_node **ltable=(ht_tree_node**)malloc(sizeof (ht_tree_node*) * node_count);
+		ht_tree_node **l=ltable;
+		/* create ltable from tree */
+		populate_ltable(&l, root);
+		if (compare_keys != new_compare_keys) {
+			/* set new keying method */
+			compare_keys = new_compare_keys;
+			/* re-sort ltable */
+			qsort_compare_compare_keys = compare_keys;
+			qsort(ltable, node_count, sizeof *ltable, qsort_compare_keys_tree_node);
+		}
+		/* rebuild tree from ltable and old_tree */
+		insert_ltable(&root, ltable, ltable+node_count-1);
+		/* destroy ltable */
+		free(ltable);
 	}
-	ht_tree::done();
 }
 
 void ht_stree::balance()
 {
-/* FIXME: debug */
-	if (node_count) {
-		ht_tree_node **ltable=(ht_tree_node**)malloc(sizeof (ht_tree_node*) * node_count);
-		ht_tree_node **l=ltable;
-/* create ltable from tree */
-		populate_ltable(&l, root);
-/* save old tree, empty tree */
-		UINT old_node_count=node_count;
-		ht_tree_node *old_root=root;
-		node_count=0;
-		root=0;
-/* rebuild tree from ltable and old_tree */
-		insert_ltable(ltable, ltable+old_node_count-1);
-/* destroy old_tree */
-		node_count=old_node_count;
-		free_skeleton(old_root);
-/* destroy ltable */
-		free(ltable);
-/**/
-		node_count=old_node_count;
-	}
+	set_compare_keys(compare_keys);
 }
 
 UINT ht_stree::count()
@@ -260,13 +286,21 @@ bool ht_stree::del(ht_data *key)
 		} else {
 			pp=&root;
 		}
-		*pp=0;
+		*pp=NULL;
 		delete node;
 	}
 	node_count--;
 	return true;
 }
 
+void ht_stree::empty()
+{
+	if (root) {
+		free_all(root);
+		root=NULL;
+	}
+}
+
 void ht_stree::enum_next_i(ht_tree_node *node, ht_data *prevkey, ht_tree_node **retv)
 {
 	if ((prevkey == NULL) || (compare_keys(prevkey, node->key) < 0)) {
@@ -289,6 +323,31 @@ void ht_stree::enum_prev_i(ht_tree_node *node, ht_data *nextkey, ht_tree_node **
 
 ht_data *ht_stree::enum_next(ht_data **value, ht_data *prevkey)
 {
+#ifdef NEW_TREE_ENUM
+	ht_tree_node *n = NULL, *next = NULL;
+	if (root) {
+		if (prevkey) {
+			n = root;
+			while (n) {
+				int c = compare_keys(prevkey, n->key);
+				if (c>0) {
+					n = n->right;
+				} else if (c<0) {
+					next = n;
+					n = n->left;
+				} else {
+					if (n->right) next = get_leftmost_node(n->right);
+					break;
+				}
+			}
+		} else next = get_leftmost_node(root);
+	}
+	if (next) {
+		*value = next->value;
+		return next->key;
+	}
+	return NULL;
+#else
 	ht_tree_node *n = NULL;
 	if (root) enum_next_i(root, prevkey, &n);
 	if (n) {
@@ -296,10 +355,36 @@ ht_data *ht_stree::enum_next(ht_data **value, ht_data *prevkey)
 		return n->key;
 	}
 	return NULL;
+#endif
 }
 
 ht_data *ht_stree::enum_prev(ht_data **value, ht_data *nextkey)
 {
+#ifdef NEW_TREE_ENUM
+	ht_tree_node *n = NULL, *prev = NULL;
+	if (root) {
+		if (nextkey) {
+			n = root;
+			while (n) {
+				int c = compare_keys(nextkey, n->key);
+				if (c>0) {
+					prev = n;
+					n = n->right;
+				} else if (c<0) {
+					n = n->left;
+				} else {
+					if (n->left) prev = get_rightmost_node(n->left);
+					break;
+				}
+			}
+		} else prev = get_rightmost_node(root);
+	}
+	if (prev) {
+		*value = prev->value;
+		return prev->key;
+	}
+	return NULL;
+#else
 	ht_tree_node *n = NULL;
 	if (root) enum_prev_i(root, nextkey, &n);
 	if (n) {
@@ -307,6 +392,7 @@ ht_data *ht_stree::enum_prev(ht_data **value, ht_data *nextkey)
 		return n->key;
 	}
 	return NULL;
+#endif
 }
 
 void ht_stree::free_all(ht_tree_node *node)
@@ -321,7 +407,7 @@ void ht_stree::free_all(ht_tree_node *node)
 		delete node->value;
 	}		
 
-	free(node);
+	delete node;
 	node_count--;
 }
 
@@ -329,7 +415,7 @@ void ht_stree::free_skeleton(ht_tree_node *node)
 {
 	if (node->left) free_skeleton(node->left);
 	if (node->right) free_skeleton(node->right);
-	free(node);
+	delete node;
 	node_count--;
 }
 
@@ -337,7 +423,7 @@ ht_data *ht_stree::get(ht_data *key)
 {
 	ht_tree_node *n=get_node_i(key);
 	if (n) return n->value;
-	return 0;
+	return NULL;
 }
 
 ht_tree_node *ht_stree::get_node_i(ht_data *key)
@@ -349,12 +435,12 @@ ht_tree_node *ht_stree::get_node_i(ht_data *key)
 		if (c<0) n=n->left; else
 			return n;
 	}
-	return 0;
+	return NULL;
 }
 
 bool ht_stree::get_node_and_parent(ht_data *key, ht_tree_node **node, ht_tree_node **parent_node, int *direction)
 {
-	ht_tree_node *p=0, *n=root;
+	ht_tree_node *p=NULL, *n=root;
 	int pc=0;
 	while (n) {
 		int c=compare_keys(key, n->key);
@@ -390,7 +476,8 @@ ht_tree_node *ht_stree::get_rightmost_node(ht_tree_node *node)
 
 bool ht_stree::insert(ht_data *key, ht_data *value)
 {
-	if ((!key) || (!value)) return false;
+//	if ((!key) || (!value)) return false;
+	if (!key) return false;
 	ht_tree_node **n=&root;
 	while (*n) {
 		int c=compare_keys(key, (*n)->key);
@@ -401,61 +488,88 @@ bool ht_stree::insert(ht_data *key, ht_data *value)
 	*n=new ht_tree_node();
 	(*n)->key=key;
 	(*n)->value=value;
-	(*n)->left=0;
-	(*n)->right=0;
+	(*n)->left=NULL;
+	(*n)->right=NULL;
 	node_count++;
 	return true;
 }
 
-void ht_stree::insert_ltable(ht_tree_node **start, ht_tree_node **end)
+void ht_stree::insert_ltable(ht_tree_node **node, ht_tree_node **start, ht_tree_node **end)
 {
-	if (start<=end) {
-		ht_tree_node **m=start+(end-start)/2;
-		ht_stree::insert((*m)->key, (*m)->value);
-		insert_ltable(start, m-1);
-		insert_ltable(m+1, end);
+	if (start <= end) {
+		ht_tree_node **m = start+(end-start)/2;
+		*node = *m;
+		insert_ltable(&(*node)->left, start, m-1);
+		insert_ltable(&(*node)->right, m+1, end);
+	} else {
+		*node = NULL;
 	}
 }
 
+void stree_load(ht_object_stream *s, ht_tree_node **n, UINT *node_count, int l, int r)
+{
+	if (l>r) {
+		*n = NULL;
+		return;
+	}
+	*n = new ht_tree_node();
+	int m = (l+r)/2;
+	stree_load(s, &(*n)->left, node_count, l, m-1);
+
+	(*n)->key = s->getObject(NULL);
+	(*n)->value = s->getObject(NULL);
+	(*node_count)++;
+
+	stree_load(s, &(*n)->right, node_count, m+1, r);
+}
+
 int ht_stree::load(ht_object_stream *s)
 {
-	void *d=find_atom(s->get_int_hex(4, 0));
+	void *d=find_atom(s->getIntHex(4, NULL));
 	if (!d) return 1;
 	compare_keys=(compare_keys_func_ptr)d;
 	
-	UINT c=s->get_int_dec(4, 0);
-
-	root=0;
+	UINT c=s->getIntDec(4, NULL);
+#if 0
+	root=NULL;
 	node_count=0;
 	
 	for (UINT i=0; i<c; i++) {
-		ht_data *key=s->get_object(0);
-		ht_data *value=s->get_object(0);
+		ht_data *key=s->get_object(NULL);
+		ht_data *value=s->get_object(NULL);
 		if (!key) return 1;
 		if (!value) return 1;
 		insert(key, value);
 	}
 	balance();
+#else
+	root=NULL;
+	node_count=0;
+
+	stree_load(s, &root, &node_count, 0, c-1);
+	assert(node_count == c);
+	
+#endif
 	return s->get_error();
 }
 
 void ht_stree::store(ht_object_stream *s)
 {
-	s->put_int_hex(find_atom_rev((void*)compare_keys), 4, 0);
+	s->putIntHex(find_atom_rev((void*)compare_keys), 4, NULL);
 	
-	s->put_int_dec(count(), 4, 0);
+	s->putIntDec(count(), 4, NULL);
 	
 	ht_data *key=NULL;
 	ht_data *value;
 	while ((key=enum_next(&value, key))) {
 		if (value) {
-			s->put_object(key, 0);
-			s->put_object(value, 0);
+			s->putObject(key, NULL);
+			s->putObject(value, NULL);
 		}
 	}
 }
 
-OBJECT_ID ht_stree::object_id()
+OBJECT_ID ht_stree::object_id() const
 {
 	return ATOM_HT_STREE;
 }
@@ -463,10 +577,8 @@ OBJECT_ID ht_stree::object_id()
 void ht_stree::populate_ltable(ht_tree_node ***ltable, ht_tree_node *node)
 {
 	if (node->left) populate_ltable(ltable, node->left);
-	if (node->value) {
-		**ltable=node;
-		(*ltable)++;
-	}
+	**ltable=node;
+	(*ltable)++;
 	if (node->right) populate_ltable(ltable, node->right);
 }
 
@@ -489,38 +601,60 @@ void ht_dtree::done()
 	ht_stree::done();
 }
 
-void ht_dtree::balance()
+void ht_dtree::populate_ltable(ht_tree_node ***ltable, ht_tree_node *node)
+{
+	if (node->left) populate_ltable(ltable, node->left);
+	if (node->value) {
+		**ltable=node;
+		(*ltable)++;
+	}
+	if (node->right) populate_ltable(ltable, node->right);
+}
+
+void ht_dtree::populate_ltable_free_dead_nodes(ht_tree_node ***ltable, ht_tree_node *node)
+{
+	if (node->left) populate_ltable(ltable, node->left);
+	if (node->value) {
+		**ltable=node;
+		(*ltable)++;
+	} else {
+		delete node;
+	}
+	if (node->right) populate_ltable(ltable, node->right);
+}
+
+void ht_dtree::set_compare_keys(compare_keys_func_ptr new_compare_keys)
 {
 	ub_delete=0;
 	ub_insert=0;
 	if (node_count-dead_node_count) {
-/* create ltable from tree */
-		UINT new_node_count=node_count-dead_node_count;
-		ht_tree_node **ltable=(ht_tree_node**)malloc(sizeof (ht_tree_node*) * new_node_count);
-		ht_tree_node **l=ltable;
-		populate_ltable(&l, root);
-/* FIXME: debuggg !!! */		
-/*		if (l-ltable!=new_node_count) {
-			fprintf(stderr, "something went wrong...\n");
-			fprintf(stderr, "fd=%d, nc=%d, dnc=%d, nnc=%d\n", l-ltable, node_count, dead_node_count, new_node_count);
-			exit(1);
-		}*/
-		
-/* save old tree, empty tree */
-		UINT old_node_count=node_count;
-		ht_tree_node *old_root=root;
-		node_count=0;
-		root=0;
-/* rebuild tree from ltable and old_tree */
-		insert_ltable(ltable, ltable+new_node_count-1);
-/* destroy old_tree */
-		node_count=old_node_count;
-		free_skeleton(old_root);
-/* destroy ltable */
+		/* create ltable from tree */
+		UINT new_node_count = node_count-dead_node_count;
+		ht_tree_node **ltable = (ht_tree_node**)malloc(sizeof (ht_tree_node*) * new_node_count);
+		assert(ltable);
+		ht_tree_node **l = ltable;
+		populate_ltable_free_dead_nodes(&l, root);
+
+		/* save old tree, empty tree */
+		UINT old_node_count = node_count;
+		node_count = 0;
+		root = NULL;
+		if (compare_keys != new_compare_keys) {
+			/* set new keying method */
+			compare_keys = new_compare_keys;
+			/* re-sort ltable */
+			qsort_compare_compare_keys = compare_keys;
+			qsort(ltable, new_node_count, sizeof *ltable, qsort_compare_keys_tree_node);
+		}
+		/* rebuild tree from ltable and old_tree */
+		insert_ltable(&root, ltable, ltable+new_node_count-1);
+		/* destroy old_tree */
+		node_count = old_node_count;
+		/* destroy ltable */
 		free(ltable);
-/**/
-		node_count=new_node_count;
-		dead_node_count=0;
+		/**/
+		node_count = new_node_count;
+		dead_node_count = 0;
 	}
 }
 
@@ -535,7 +669,7 @@ bool ht_dtree::del(ht_data *key)
 	if ((n) && (n->value)) {
 		n->value->done();
 		delete n->value;
-		n->value=0;
+		n->value=NULL;
 		dead_node_count++;
 		if (max_ub_delete) {
 			if (++ub_delete>max_ub_delete) balance();
@@ -547,6 +681,36 @@ bool ht_dtree::del(ht_data *key)
 
 ht_data *ht_dtree::enum_next(ht_data **value, ht_data *prevkey)
 {
+#ifdef NEW_TREE_ENUM
+	ht_tree_node *n, *next;
+	do {
+		n = NULL;
+		next = NULL;
+	if (root) {
+		if (prevkey) {
+			n = root;
+			while (n) {
+				int c = compare_keys(prevkey, n->key);
+				if (c>0) {
+					n = n->right;
+				} else if (c<0) {
+					next = n;
+					n = n->left;
+				} else {
+					if (n->right) next = get_leftmost_node(n->right);
+					break;
+				}
+			}
+		} else next = get_leftmost_node(root);
+	}
+		if (next) prevkey = next->key;
+	} while (next && !next->value);
+	if (next) {
+		*value = next->value;
+		return next->key;
+	}
+	return NULL;
+#else
 	ht_tree_node *n;
 	ht_data *k = prevkey;
 	while (1) {
@@ -560,10 +724,41 @@ ht_data *ht_dtree::enum_next(ht_data **value, ht_data *prevkey)
 		k = n->key;
 	}
 	return NULL;
+#endif
 }
 
 ht_data *ht_dtree::enum_prev(ht_data **value, ht_data *nextkey)
 {
+#ifdef NEW_TREE_ENUM
+	ht_tree_node *n, *prev;
+	while (1) {
+	n = NULL;
+	prev = NULL;
+	if (root) {
+		if (nextkey) {
+			n = root;
+			while (n) {
+				int c = compare_keys(nextkey, n->key);
+				if (c>0) {
+					prev = n;
+					n = n->right;
+				} else if (c<0) {
+					n = n->left;
+				} else {
+					if (n->left) prev = get_rightmost_node(n->left);
+					break;
+				}
+			}
+		} else prev = get_rightmost_node(root);
+	}
+		if (prev) nextkey = prev->key;
+	} while (prev && !prev->value);
+	if (prev) {
+		*value = prev->value;
+		return prev->key;
+	}
+	return NULL;
+#else
 	ht_tree_node *n;
 	ht_data *k = nextkey;
 	while (1) {
@@ -577,6 +772,7 @@ ht_data *ht_dtree::enum_prev(ht_data **value, ht_data *nextkey)
 		k = n->key;
 	}
 	return NULL;
+#endif
 }
 
 bool ht_dtree::insert(ht_data *key, ht_data *value)
@@ -600,8 +796,8 @@ bool ht_dtree::insert(ht_data *key, ht_data *value)
 		*n=new ht_tree_node();
 		(*n)->key=key;
 		(*n)->value=value;
-		(*n)->left=0;
-		(*n)->right=0;
+		(*n)->left=NULL;
+		(*n)->right=NULL;
 		node_count++;
 	}
 	if (max_ub_insert) {
@@ -627,7 +823,7 @@ void ht_list::done()
 
 void ht_list::destroy()
 {
-	ht_list::done();
+	done();
 }
 
 void ht_list::append(ht_data *data)
@@ -655,7 +851,7 @@ void ht_list::copy_to(UINT i, UINT count, ht_list *destlist)
 
 ht_list *ht_list::cut(UINT i, UINT count)
 {
-	return 0;
+	return NULL;
 }
 
 bool ht_list::del(UINT i)
@@ -683,7 +879,7 @@ UINT ht_list::find(ht_data *data)
 
 ht_data *ht_list::get(UINT i)
 {
-	return 0;
+	return NULL;
 }
 
 void ht_list::insert(ht_data *data)
@@ -712,14 +908,14 @@ void ht_list::prepend(ht_data *data)
 
 ht_data *ht_list::remove(UINT i)
 {
-	return 0;
+	return NULL;
 }
 
 bool ht_list::remove_multiple(UINT i, UINT count)
 {
 	bool b=true;
 	while (count--) {
-		b&=(remove(i)!=0);
+		b&=(remove(i)!=NULL);
 	}
 	return b;
 }
@@ -738,13 +934,16 @@ bool ht_list::sort()
  *	CLASS ht_clist
  */
 
-#define HT_CLIST_ENTRY_COUNT_BASE		32
-#define HT_CLIST_ENTRY_COUNT_EXTEND	64
+#define HT_CLIST_ENTRY_COUNT_START		32
+
+// exponential growth factor: num/den
+#define HT_CLIST_ENTRY_COUNT_EXT_NUM	3
+#define HT_CLIST_ENTRY_COUNT_EXT_DEN	2
 
 void ht_clist::init(compare_keys_func_ptr compare_keys)
 {
 	ht_list::init(compare_keys);
-	c_size=HT_CLIST_ENTRY_COUNT_BASE;
+	c_size=HT_CLIST_ENTRY_COUNT_START;
 	items=(ht_data**)malloc(c_size * sizeof (ht_data*));
 	c_entry_count=0;
 }
@@ -757,10 +956,8 @@ void ht_clist::done()
 
 void ht_clist::destroy()
 {
-	for (UINT i=0; i<c_entry_count; i++) {
-		do_free(i);
-	}
-	ht_clist::done();
+	empty();
+	done();
 }
 
 void ht_clist::append(ht_data *data)
@@ -808,7 +1005,7 @@ void ht_clist::do_remove(UINT i)
 	c_entry_count--;
 }
 
-object *ht_clist::duplicate()
+Object *ht_clist::duplicate()
 {
 	ht_clist *d=new ht_clist();
 	d->init(compare_keys);
@@ -828,7 +1025,8 @@ void ht_clist::empty()
 
 void ht_clist::extend_list()
 {
-	c_size+=HT_CLIST_ENTRY_COUNT_EXTEND;
+	c_size *= HT_CLIST_ENTRY_COUNT_EXT_NUM;
+	c_size /= HT_CLIST_ENTRY_COUNT_EXT_DEN;
 	ht_data **new_items=(ht_data**)malloc(c_size * sizeof (ht_data*));
 	memmove(new_items, items, c_entry_count * sizeof (ht_data*));
 	free(items);
@@ -848,7 +1046,7 @@ UINT ht_clist::find(ht_data *data)
 ht_data *ht_clist::get(UINT i)
 {
 	if (i<c_entry_count) return items[i];
-	return 0;
+	return NULL;
 }
 
 void ht_clist::insert(ht_data *data)
@@ -876,17 +1074,17 @@ void ht_clist::insert_before(ht_data *data, UINT i)
 
 int  ht_clist::load(ht_object_stream *s)
 {
-	c_size=HT_CLIST_ENTRY_COUNT_BASE;
+	c_size=HT_CLIST_ENTRY_COUNT_START;
 	items=(ht_data**)malloc(c_size * sizeof (ht_data*));
 	c_entry_count=0;
 	
-	void *d=find_atom(s->get_int_hex(4, 0));
+	void *d=find_atom(s->getIntHex(4, NULL));
 //	if (!d) return 1;
 	compare_keys=(compare_keys_func_ptr)d;
 	
-	int c=s->get_int_dec(4, "item_count");
+	int c=s->getIntDec(4, "item_count");
 	for (int i=0; i<c; i++) {
-		ht_data *d=s->get_object("item");
+		ht_data *d=s->getObject("item");
 		if (s->get_error()) break;
 		prepend(d);
 	}
@@ -912,7 +1110,7 @@ void ht_clist::move_multiple(UINT source, UINT dest, UINT count)
 	}
 }
 
-OBJECT_ID ht_clist::object_id()
+OBJECT_ID ht_clist::object_id() const
 {
 	return ATOM_HT_CLIST;
 }
@@ -929,43 +1127,32 @@ ht_data *ht_clist::remove(UINT i)
 		do_remove(i);
 		return d;
 	}
-	return 0;
+	return NULL;
 }
 
 bool ht_clist::set(UINT i, ht_data *data)
 {
-	if (i<c_entry_count) {
-		do_free(i);
-		items[i]=data;
-		return true;
-	}
-	return false;
-}
-
-int (*clist_sort_compare_compare_keys)(ht_data *key_a, ht_data *key_b);
-
-int clist_sort_compare(const void *e1, const void *e2)
-{
-	ht_data **d1=(ht_data **)e1;
-	ht_data **d2=(ht_data **)e2;
-	return clist_sort_compare_compare_keys(*d1, *d2);
+	while (i>=c_entry_count) append(NULL);
+	do_free(i);
+	items[i] = data;
+	return true;
 }
 
 bool ht_clist::sort()
 {
 	if (compare_keys && (c_entry_count>1)) {
-		clist_sort_compare_compare_keys=compare_keys;
-		qsort(items, c_entry_count, sizeof *items, clist_sort_compare);
+		qsort_compare_compare_keys = compare_keys;
+		qsort(items, c_entry_count, sizeof *items, qsort_compare);
 	}
 	return false;
 }
 
 void ht_clist::store(ht_object_stream *s)
 {
-	s->put_int_hex(find_atom_rev((void*)compare_keys), 4, 0);
-	s->put_int_dec(c_entry_count, 4, "item_count");
+	s->putIntHex(find_atom_rev((void*)compare_keys), 4, NULL);
+	s->putIntDec(c_entry_count, 4, "item_count");
 	for (int i=c_entry_count-1; i>=0; i--) {
-		s->put_object(items[i], "item");
+		s->putObject(items[i], "item");
 	}
 }
 
@@ -1098,7 +1285,7 @@ ht_data *ht_stack::pop()
 		do_remove(c_entry_count-1);
 		return d;
 	}
-	return 0;
+	return NULL;
 }
 
 void ht_stack::push(ht_data *data)
@@ -1122,7 +1309,7 @@ ht_data *ht_queue::dequeue()
 		do_remove(0);
 		return d;
 	}
-	return 0;
+	return NULL;
 }
 
 ht_data *ht_queue::pop()
@@ -1147,13 +1334,18 @@ char *matchhash(int value, int_hash *hash_table)
 			hash_table++;
 		}
 	}
-	return 0;
+	return NULL;
 }
 
 /*
  *	compare procedures
  */
 
+int compare_keys_ht_data(ht_data *key_a, ht_data *key_b)
+{
+	return key_a->compareTo(key_b);
+}
+
 int compare_keys_int(ht_data *key_a, ht_data *key_b)
 {
 	int a=((ht_data_uint*)key_a)->value;
@@ -1188,6 +1380,7 @@ bool init_data()
 	REGISTER(ATOM_HT_STREE, ht_stree);
 	REGISTER(ATOM_HT_CLIST, ht_clist);
 	
+	register_atom(ATOM_COMPARE_KEYS_HT_DATA, (void*)compare_keys_ht_data);
 	register_atom(ATOM_COMPARE_KEYS_INT, (void*)compare_keys_int);
 	register_atom(ATOM_COMPARE_KEYS_UINT, (void*)compare_keys_uint);
 	
@@ -1200,6 +1393,7 @@ bool init_data()
 
 void done_data()
 {
+	unregister_atom(ATOM_COMPARE_KEYS_HT_DATA);
 	unregister_atom(ATOM_COMPARE_KEYS_INT);
 	unregister_atom(ATOM_COMPARE_KEYS_UINT);
 	
