@@ -2,7 +2,7 @@
  *	HT Editor
  *	htelfimg.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,8 +21,22 @@
 #ifndef __HTELFIMG_H__
 #define __HTELFIMG_H__
 
+#include "htanaly.h"
+#include "htelf.h"
 #include "formats.h"
 
 extern format_viewer_if htelfimage_if;
 
+/*
+ *	CLASS ht_elf_aviewer
+ */
+
+class ht_elf_aviewer: public ht_aviewer {
+public:
+	ht_elf_shared_data *elf_shared;
+		   void init(bounds *b, char *desc, int caps, ht_streamfile *file, ht_format_group *format_group, Analyser *Analyser, ht_elf_shared_data *elf_shared);
+	virtual void setAnalyser(Analyser *a);
+};
+
 #endif /* !__HTELFIMG_H__ */
+
