@@ -2,7 +2,7 @@
  *	HT Editor
  *	htidle.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -22,8 +22,8 @@
 #define HTIDLE_H
 
 #include "common.h"
-void register_idle_object(object *o);
-void unregister_idle_object(object *o);
+void register_idle_object(Object *o);
+void unregister_idle_object(Object *o);
 void do_idle();
 
 /*
