@@ -2,7 +2,7 @@
  *	HTE Editor
  *	htinfo.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -23,22 +23,15 @@
 
 #define ht_url "http://hte.sourceforge.net"
 
-/*
- *   HT
- */
-
 #define ht_name "ht"
-#define ht_version "0.5.0"
-#define ht_copyright "(c) 1999, 2000, 2001 "ht_author" ("ht_mail")"
-#define ht_loadtitle ht_name" "ht_version" "ht_copyright
+#define ht_version "0.7.3"
 
-#define ht_author "Stefan Weyergraf"
-#define ht_mail "stefan@weyergraf.de"
+#define ht_author1 "Stefan Weyergraf"
+#define ht_mail1 "stefan@weyergraf.de"
+#define ht_copyright1 "(c) 1999-2003 "ht_author1", "ht_mail1
 
-/*
- *   HT - analyser
- */
-
-#include "analyinfo.h"
+#define ht_author2 "Sebastian Biallas"
+#define ht_mail2 "sb@biallas.net"
+#define ht_copyright2 "(c) 1999-2003 "ht_author2", "ht_mail2
 
 #endif /* !__HTINFO_H__ */
