@@ -2,7 +2,7 @@
  *	HT Editor
  *	htleimg.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -21,33 +21,29 @@
 #ifndef __HTLEIMG_H__
 #define __HTLEIMG_H__
 
-#include "stream.h"
-
+#include "htanaly.h"
+#include "htle.h"
 #include "formats.h"
 
-/*
- *	CLASS ht_le_page_file
- */
-
-class ht_le_page_file: public ht_layer_streamfile {
-protected:
-	ht_le_pagemap *pagemap;
-	dword pagemapsize;
-	dword page_size;
-	FILEOFS ofs;
-/* new */
-	int map_ofs(dword ofs, FILEOFS *offset, dword *maxsize);
-public:
-		void init(ht_streamfile *file, bool own_file, ht_le_pagemap *pagemap, dword pagemapsize, dword page_size);
-/* overwritten */
-	bool isdirty(FILEOFS offset, UINT range);
-	UINT read(void *buf, dword size);
-	int seek(FILEOFS offset);
-	FILEOFS tell();
-	UINT write(void *buf, UINT size);
-};
-
 extern format_viewer_if htleimage_if;
 
+/*
+ *	CLASS ht_le_aviewer
+ */
+
+class ht_le_aviewer: public ht_aviewer {
+public:
+	ht_le_shared_data *le_shared;
+	ht_streamfile *file;
+	
+		   void init(bounds *b, char *desc, int caps, ht_streamfile *file, ht_format_group *format_group, Analyser *Analyser, ht_le_shared_data *le_shared);
+/* overwritten */
+	virtual char *func(UINT i, bool execute);
+	virtual void setAnalyser(Analyser *a);
+	virtual bool get_current_real_offset(FILEOFS *ofs);
+	virtual bool offset_to_pos(FILEOFS ofs, viewer_pos *p);
+	virtual bool pos_to_offset(viewer_pos p, FILEOFS *ofs);
+};
+
 #endif /* !__HTLEIMG_H__ */
 
