@@ -2,7 +2,7 @@
  *	HT Editor
  *	htneent.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -25,4 +25,14 @@
 
 extern format_viewer_if htneentrypoints_if;
 
+/*
+ *	CLASS ht_ne_entrypoint_viewer
+ */
+
+class ht_ne_entrypoint_viewer: public ht_uformat_viewer {
+public:
+/* overwritten */
+	virtual	int ref_sel(LINE_ID *id);
+};
+
 #endif /* !__HTNEENT_H__ */
