@@ -2,7 +2,7 @@
  *	HT Editor
  *	htpehead.h
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -25,4 +25,15 @@
 
 extern format_viewer_if htpeheader_if;
 
+/*
+ *	CLASS ht_pe_header_viewer
+ */
+
+class ht_pe_header_viewer: public ht_uformat_viewer {
+public:
+			void init(bounds *b, char *desc, int caps, ht_streamfile *file, ht_format_group *group);
+/* overwritten */
+	virtual	int ref_sel(LINE_ID *id);
+};
+
 #endif /* !__HTPEHEAD_H__ */
