@@ -2,7 +2,7 @@
  *	HT Editor
  *	htsearch.cc
  *
- *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -18,8 +18,8 @@
  *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "htapp.h"
 #include "htatom.h"
+#include "cmds.h"
 #include "htctrl.h"
 #include "hthist.h"
 #include "htiobox.h"
@@ -27,23 +27,22 @@
 #include "htsearch.h"
 #include "htstring.h"
 #include "process.h"
+#include "snprintf.h"
 
 extern "C" {
 #include "evalx.h"
 }
 
+union search_pos {
+	FILEOFS offset;
+	viewer_pos pos;
+};
+
 /* FIXME: get rid of global vars */
 UINT lastsearchmodeid=0;
 UINT lastreplacemodeid=0;
 
-union search_pos {
-	FILEOFS offset;
-	fmt_vaddress address;
-};
-
-typedef ht_view* (*create_form_func)(bounds *b, HT_ATOM histid);
 typedef ht_search_request* (*create_request_func)(search_pos *ret_start, search_pos *ret_end, ht_view *form, ht_format_viewer *format, UINT search_class);
-typedef void (*create_desc_func)(char *buf, int buflen, ht_view *form);
 
 typedef ht_data* (*create_replace_context_func)(ht_streamfile *file, FILEOFS ofs, UINT len, ht_view *form, UINT *return_repllen);
 
@@ -86,17 +85,17 @@ bool test_str_to_ofs(FILEOFS *ofs, byte *str, UINT strlen, ht_format_viewer *for
 	return true;
 }
 
-bool test_str_to_addr(fmt_vaddress *addr, byte *str, UINT strlen, ht_format_viewer *format, char *desc)
+bool test_str_to_pos(viewer_pos *pos, byte *str, UINT strlen, ht_format_viewer *format, char *desc)
 {
-#define TEST_STR_TO_ADDR_MAXSTRLEN      128
-	if (strlen>TEST_STR_TO_ADDR_MAXSTRLEN) {
-		throw new ht_io_exception("%s: expression too long (len %d, max %d)", desc, strlen, TEST_STR_TO_ADDR_MAXSTRLEN);
+#define TEST_STR_TO_POS_MAXSTRLEN      128
+	if (strlen>TEST_STR_TO_POS_MAXSTRLEN) {
+		throw new ht_io_exception("%s: expression too long (len %d, max %d)", desc, strlen, TEST_STR_TO_POS_MAXSTRLEN);
 		return false;
 	}
 	if (strlen>0) {
-		char s[TEST_STR_TO_ADDR_MAXSTRLEN+1];
+		char s[TEST_STR_TO_POS_MAXSTRLEN+1];
 		bin2str(s, str, strlen);
-		if (!format->string_to_address(s, addr)) {
+		if (!format->string_to_pos(s, pos)) {
 			throw new ht_io_exception("%s: invalid expression: '%s'", desc, s);
 			return false;
 		}
@@ -119,7 +118,7 @@ ht_search_request* create_request_hexascii(search_pos *start, search_pos *end, h
 {
 	ht_hexascii_search_form *form=(ht_hexascii_search_form*)f;
 	ht_hexascii_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
 	
 	ht_fxbin_search_request *request;
 	
@@ -141,7 +140,11 @@ void create_desc_hexascii(char *buf, int buflen, ht_view *f)
 {
 	ht_hexascii_search_form *form=(ht_hexascii_search_form*)f;
 	ht_hexascii_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
+
+	if (!d.str.textlen) {
+		throw new ht_io_exception("%s: string is empty", "hex/ascii");
+	}
 
 	char *b = buf;
 	b += escape_special(b, buflen, d.str.text, d.str.textlen, NULL, false);
@@ -178,7 +181,7 @@ ht_search_request* create_request_evalstr(search_pos *start, search_pos *end, ht
 #define EVALSTR_MAXSTRLEN		256
 	ht_evalstr_search_form *form=(ht_evalstr_search_form*)f;
 	ht_evalstr_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
 	
 	ht_fxbin_search_request *request = NULL;
 		
@@ -188,8 +191,8 @@ ht_search_request* create_request_evalstr(search_pos *start, search_pos *end, ht
 		char strbuf[EVALSTR_MAXSTRLEN+1];
 		bin2str(strbuf, d.str.text, d.str.textlen);
 
-		scalar_t r;
-		str_t s;
+		eval_scalar r;
+		eval_str s;
 		if (eval(&r, strbuf, NULL, NULL, NULL)) {
 			scalar_context_str(&r, &s);
 			scalar_destroy(&r);
@@ -215,7 +218,11 @@ void create_desc_evalstr(char *buf, int buflen, ht_view *f)
 {
 	ht_evalstr_search_form *form=(ht_evalstr_search_form*)f;
 	ht_evalstr_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
+
+	if (!d.str.textlen) {
+		throw new ht_io_exception("%s: string is empty", "eval str");
+	}
 
 	char *b = buf;
 	b += escape_special(b, buflen, d.str.text, d.str.textlen, NULL, false);
@@ -243,8 +250,8 @@ void create_desc_evalstr(char *buf, int buflen, ht_view *f)
 ht_fxbin_search_request::ht_fxbin_search_request(UINT search_class, UINT flags, UINT ds, byte *d) :
 	ht_search_request(search_class, ST_FXBIN, flags)
 {
-	data_size=ds;
-	data=(byte*)malloc(data_size);
+	data_size = ds;
+	data = (byte*)malloc(data_size);
 	memmove(data, d, data_size);
 }
 
@@ -253,11 +260,9 @@ ht_fxbin_search_request::~ht_fxbin_search_request()
 	free(data);
 }
 
-object *ht_fxbin_search_request::duplicate()
+Object *ht_fxbin_search_request::duplicate()
 {
-	byte *dt=(byte*)malloc(data_size);
-	memmove(dt, data, data_size);
-	return new ht_fxbin_search_request(search_class, flags, data_size, dt);
+	return new ht_fxbin_search_request(search_class, flags, data_size, data);
 }
 
 /*
@@ -276,7 +281,7 @@ ht_search_request* create_request_vregex(search_pos *start, search_pos *end, ht_
 #define VREGEX_MAXSTRLEN		256
 	ht_vregex_search_form *form=(ht_vregex_search_form*)f;
 	ht_vregex_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
 
 	ht_regex_search_request *request=NULL;
 
@@ -286,8 +291,8 @@ ht_search_request* create_request_vregex(search_pos *start, search_pos *end, ht_
 		char strbuf[VREGEX_MAXSTRLEN+1];
 		bin2str(strbuf, d.str.text, d.str.textlen);
 
-		if (test_str_to_addr(&start->address, d.start.text, d.start.textlen, format, "start-address")
-		&& test_str_to_addr(&end->address, d.end.text, d.end.textlen, format, "end-address")) {
+		if (test_str_to_pos(&start->pos, d.start.text, d.start.textlen, format, "start-address")
+		&& test_str_to_pos(&end->pos, d.end.text, d.end.textlen, format, "end-address")) {
 			request = new ht_regex_search_request(search_class, d.options.state & 1 ? SF_REGEX_CASEINSENSITIVE : 0, strbuf);
 		}
 	} else {
@@ -300,7 +305,11 @@ void create_desc_vregex(char *buf, int buflen, ht_view *f)
 {
 	ht_vregex_search_form *form=(ht_vregex_search_form*)f;
 	ht_vregex_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
+
+	if (!d.str.textlen) {
+		throw new ht_io_exception("%s: string is empty", "regex");
+	}
 
 	char *b = buf;
 	b += escape_special(b, buflen, d.str.text, d.str.textlen, NULL, false);
@@ -328,17 +337,18 @@ void create_desc_vregex(char *buf, int buflen, ht_view *f)
 ht_regex_search_request::ht_regex_search_request(UINT search_class, UINT flags, char *regex) :
 	ht_search_request(search_class, ST_REGEX, flags)
 {
-	rx_str=ht_strdup(regex);
-	int r=regcomp(&rx, rx_str, REG_EXTENDED | ((flags & SF_REGEX_CASEINSENSITIVE) ? REG_ICASE : 0));
+	rx_str = ht_strdup(regex);
+	int r = regcomp(&rx, rx_str, REG_EXTENDED | ((flags & SF_REGEX_CASEINSENSITIVE) ? REG_ICASE : 0));
 	if (r) throw new ht_regex_search_exception(r, &rx);
 }
 
 ht_regex_search_request::~ht_regex_search_request()
 {
 	regfree(&rx);
+	free(rx_str);
 }
 
-object *ht_regex_search_request::duplicate()
+Object *ht_regex_search_request::duplicate()
 {
 	return new ht_regex_search_request(search_class, flags, rx_str);
 }
@@ -351,10 +361,10 @@ ht_regex_search_exception::ht_regex_search_exception(int e, regex_t *r)
 {
 	errorcode=e;
 	regex=r;
-	
+
 	char s[128];
-	regerror(errorcode, regex, s, sizeof s-32);
-	sprintf(rxerr, "error compiling regex: %s", s);
+	regerror(errorcode, regex, s, sizeof s);
+	ht_snprintf(rxerr, sizeof rxerr, "error compiling regex: %s", s);
 }
 	
 const char* ht_regex_search_exception::what()
@@ -378,7 +388,7 @@ ht_search_request* create_request_expr(search_pos *start, search_pos *end, ht_vi
 #define EXPR_MAXSTRLEN		256
 	ht_expr_search_form *form=(ht_expr_search_form*)f;
 	ht_expr_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
 	
 	ht_expr_search_request *request = NULL;
 
@@ -402,7 +412,11 @@ void create_desc_expr(char *buf, int buflen, ht_view *f)
 {
 	ht_vregex_search_form *form=(ht_vregex_search_form*)f;
 	ht_vregex_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
+
+	if (!d.str.textlen) {
+		throw new ht_io_exception("%s: string is empty", "expr");
+	}
 
 	char *b = buf;
 	b += escape_special(b, buflen, d.str.text, d.str.textlen, NULL, false);
@@ -438,7 +452,7 @@ ht_expr_search_request::~ht_expr_search_request()
 	if (expr) delete expr;
 }
 
-object *ht_expr_search_request::duplicate()
+Object *ht_expr_search_request::duplicate()
 {
 	return new ht_expr_search_request(search_class, flags, expr);
 }
@@ -447,6 +461,7 @@ object *ht_expr_search_request::duplicate()
  *	binary search
  */
 
+/* FIXME: put somewhere else */
 void bufdowncase(byte *buf, dword len)
 {
 	for (dword i=0; i<len; i++) {
@@ -456,6 +471,12 @@ void bufdowncase(byte *buf, dword len)
 
 #define SEARCH_BUF_SIZE 256*1024
 
+ht_search_bin_context::~ht_search_bin_context()
+{
+	free(buf);
+	free(pat);
+}
+
 ht_data* create_search_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, byte *pat, UINT patlen, UINT flags, UINT *return_ofs, bool *return_success)
 {
 	if (patlen > SEARCH_BUF_SIZE) return NULL;
@@ -515,10 +536,10 @@ bool search_bin_process(ht_data *context, ht_text *progress_indicator)
 	}
 	ctx->o -= ctx->patlen-1;
 
-	int p = (ctx->o - ctx->ofs)*100/ctx->len;
+	int p = (int)(((double)(ctx->o - ctx->ofs))*100/ctx->len);
 	
 	char status[64];
-	sprintf(status, "%d %%", p);
+	ht_snprintf(status, sizeof status, "%d %%", p);
 	progress_indicator->settext(status);
 	
 	return true;
@@ -537,7 +558,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 /* ascii string */
 	c.x=6;
 	c.y=0;
-	c.w=40;
+	c.w=b->w-6;
 	c.h=1;
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
@@ -553,7 +574,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 /* hex string */
 	c.x=6;
 	c.y=2;
-	c.w=40;
+	c.w=b->w-6;
 	c.h=1;
 	ht_hexinputfield *hex=new ht_hexinputfield();
 	hex->init(&c, 64);
@@ -602,15 +623,14 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	c.w=35;
 	c.y=8;
 	c.h=2;
-	ht_string_list *opts=new ht_string_list();
+	ht_string_list *opts = new ht_string_list();
 	opts->init();
-	opts->insert_string("~case insensitive");
-//	opts->insert_string("find all occurencies (~idle)");
-	option_boxes=new ht_checkboxes();
+	opts->insert_string("case ~insensitive");
+	option_boxes = new ht_checkboxes();
 	option_boxes->init(&c, opts);
 	ht_checkboxes_data d;
-	d.state=options;
-	option_boxes->databuf_set(&d);
+	d.state = options;
+	option_boxes->databuf_set(&d, sizeof d);
 	insert(option_boxes);
 }
 
@@ -627,7 +647,7 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 /* string */
 	c.x=0;
 	c.y=1;
-	c.w=40;
+	c.w=b->w;
 	c.h=1;
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
@@ -635,10 +655,10 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 /* string label */
 	c.x=0;
 	c.y=0;
-	c.w=12;
+	c.w=23;
 	c.h=1;
 	ht_label *strlabel=new ht_label();
-	strlabel->init(&c, "eval ~string", str);
+	strlabel->init(&c, "s~earch evaluated string", str);
 	insert(strlabel);
 /* hint */
 /*	c.x=0;
@@ -681,18 +701,17 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	relabel->init(&c, "~to ofs", range_end);
 	insert(relabel);
 /* options */
-	c.w=35;
-	c.y=8;
-	c.h=2;
-	ht_string_list *opts=new ht_string_list();
+	c.w = 35;
+	c.y = 8;
+	c.h = 2;
+	ht_string_list *opts = new ht_string_list();
 	opts->init();
-	opts->insert_string("~case insensitive");
-//	opts->insert_string("find all occurencies (~idle)");
-	option_boxes=new ht_checkboxes();
+	opts->insert_string("case ~insensitive");
+	option_boxes = new ht_checkboxes();
 	option_boxes->init(&c, opts);
 	ht_checkboxes_data d;
-	d.state=options;
-	option_boxes->databuf_set(&d);
+	d.state = options;
+	option_boxes->databuf_set(&d, sizeof d);
 	insert(option_boxes);
 }
 
@@ -709,7 +728,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 /* string */
 	c.x=0;
 	c.y=1;
-	c.w=40;
+	c.w=b->w;
 	c.h=1;
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
@@ -720,7 +739,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	c.w=12;
 	c.h=1;
 	ht_label *strlabel=new ht_label();
-	strlabel->init(&c, "~regular expression", str);
+	strlabel->init(&c, "s~earch regex", str);
 	insert(strlabel);
 
 /* range start */
@@ -755,18 +774,17 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	relabel->init(&c, "~to addr", range_end);
 	insert(relabel);
 /* options */
-	c.w=35;
-	c.y=8;
-	c.h=2;
-	ht_string_list *opts=new ht_string_list();
+	c.w = 35;
+	c.y = 8;
+	c.h = 2;
+	ht_string_list *opts = new ht_string_list();
 	opts->init();
-	opts->insert_string("~case insensitive");
-//	opts->insert_string("find all occurencies (~idle)");
-	option_boxes=new ht_checkboxes();
+	opts->insert_string("case ~insensitive");
+	option_boxes = new ht_checkboxes();
 	option_boxes->init(&c, opts);
 	ht_checkboxes_data d;
-	d.state=options;
-	option_boxes->databuf_set(&d);
+	d.state = options;
+	option_boxes->databuf_set(&d, sizeof d);
 	insert(option_boxes);
 }
 
@@ -774,7 +792,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
  *	CLASS ht_expr_search_form
  */
 
-void	ht_expr_search_form::init(bounds *b, int options, ht_list *history=0)
+void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 {
 	ht_group::init(b, VO_SELECTABLE, NULL);
 	VIEW_DEBUG_NAME("ht_expr_search_form");
@@ -783,7 +801,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history=0)
 /* string */
 	c.x=0;
 	c.y=1;
-	c.w=40;
+	c.w=b->w;
 	c.h=1;
 	str=new ht_strinputfield();
 	str->init(&c, 256, history);
@@ -791,10 +809,10 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history=0)
 /* string label */
 	c.x=0;
 	c.y=0;
-	c.w=12;
+	c.w=17;
 	c.h=1;
 	ht_label *strlabel=new ht_label();
-	strlabel->init(&c, "~expression", str);
+	strlabel->init(&c, "s~earch expression", str);
 	insert(strlabel);
 /* hint */
 	c.x=0;
@@ -802,7 +820,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history=0)
 	c.w=b->w-2;
 	c.h=1;
 	ht_statictext *hint=new ht_statictext();
-	hint->init(&c, "stops if expression evaluates to non-zero", 0);
+	hint->init(&c, "stops if expression evaluates to non-zero", align_left);
 	insert(hint);
 
 /* range start */
@@ -837,25 +855,28 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history=0)
 	relabel->init(&c, "~to ofs", range_end);
 	insert(relabel);
 /* options */
-	c.w=35;
+/*	c.w=35;
 	c.y=8;
 	c.h=2;
 	ht_string_list *opts=new ht_string_list();
 	opts->init();
-	opts->insert_string("~case insensitive");
-//	opts->insert_string("find all occurencies (~idle)");
 	option_boxes=new ht_checkboxes();
 	option_boxes->init(&c, opts);
 	ht_checkboxes_data d;
 	d.state=options;
 	option_boxes->databuf_set(&d);
-	insert(option_boxes);
+	insert(option_boxes);*/
 }
 
 /*
  *	CLASS ht_replace_hexascii_search_form
  */
 
+ht_replace_bin_context::~ht_replace_bin_context()
+{
+	free(repl);
+}
+
 ht_view* create_form_replace_hexascii(bounds *b, HT_ATOM histid)
 {
 	ht_replace_hexascii_search_form *form=new ht_replace_hexascii_search_form();
@@ -866,10 +887,10 @@ ht_view* create_form_replace_hexascii(bounds *b, HT_ATOM histid)
 ht_data* create_replace_hexascii_context(ht_streamfile *file, FILEOFS ofs, UINT len, ht_view *form, UINT *return_repllen)
 {
 	ht_replace_hexascii_search_form_data d;
-	form->databuf_get(&d);
+	form->databuf_get(&d, sizeof d);
 	
 	ht_replace_bin_context *ctx = (ht_replace_bin_context*)
-		create_replace_bin_context(file, ofs, len, d.str.text, d.str.textlen, return_repllen);
+	create_replace_bin_context(file, ofs, len, d.str.text, d.str.textlen, return_repllen);
 
 	return ctx;
 }
@@ -918,7 +939,7 @@ void ht_replace_hexascii_search_form::init(bounds *b, int options, ht_list *hist
  *
  */
 
-ht_search_method search_methods[] =
+static ht_search_method search_methods[] =
 {
 	{ "bin: hex/ascii", SC_PHYSICAL, SEARCHMODE_BIN, HISTATOM_SEARCH_BIN,
 		create_form_hexascii, create_request_hexascii, create_desc_hexascii },
@@ -931,15 +952,16 @@ ht_search_method search_methods[] =
 	{ NULL }
 };
 
-void search_dialog(ht_format_viewer *format, UINT searchmodes)
+ht_search_request *search_dialog(ht_format_viewer *format, UINT searchmodes, viewer_pos *start, viewer_pos *end)
 {
+	ht_search_request *result = NULL;
 	bounds b;
 	b.w = 50;
 	b.h = 15;
 	b.x = (screen->size.w-b.w)/2;
 	b.y = (screen->size.h-b.h)/2;
 	ht_search_dialog *dialog = new ht_search_dialog();
-	dialog->init(&b);
+	dialog->init(&b, "search");
 
 	bounds k;
 	dialog->search_mode_xgroup->getbounds(&k);
@@ -963,77 +985,67 @@ void search_dialog(ht_format_viewer *format, UINT searchmodes)
 	
 	dialog->select_search_mode(lastsearchmodeid);
 	
-	if (dialog->run(0)) {
+	if (dialog->run(false)) {
 		int modeid = dialog->get_search_modeid();
 		lastsearchmodeid = modeid;
 
 		ht_search_method *s = &search_methods[modeid];
 		ht_view *form = dialog->get_search_modeform();
 
-		search_pos start, end;
+		search_pos sstart, send;
 
-		ht_search_request *request = NULL;
 		try {
 /* create history entry */
 			if (s->create_desc) {
-				char hist_desc[1024];	/* FIXME: possible buffer overflow */
+				char hist_desc[1024];
 				s->create_desc(hist_desc, sizeof hist_desc, form);
 				insert_history_entry((ht_list*)find_atom(s->histid), hist_desc, form);
 			}
-/* search */
-			switch (s->search_class) {
+/* create request */
+		switch (s->search_class) {
 				case SC_PHYSICAL:
-					start.offset=0;
-					end.offset=0xffffffff;
-					format->get_current_offset(&start.offset);
+					if (!format->pos_to_offset(*start, &sstart.offset)
+						|| !format->pos_to_offset(*end, &send.offset)) {
+						throw new ht_io_exception("Internal error: can't convert viewer_pos to offset");
+					}
 					break;
 				case SC_VISUAL:
-					start.address=0;
-					end.address=0xffffffff;
-					format->get_current_address(&start.address);
+					sstart.pos = *start;
+					send.pos = *end;
 					break;
 			}
-			request = s->create_request(&start, &end, form, format, s->search_class);
-			if (request) switch (s->search_class) {
-				case SC_PHYSICAL: {
-					ht_physical_search_result *result=(ht_physical_search_result*)format->psearch(request, start.offset, end.offset);
-					if (result) {
-						format->goto_offset(result->offset, format);
-						if (result->size) {
-							format->pselect_set(result->offset, result->offset + result->size);
-						}
-						delete result;
-					} else infobox("not found");
+			result = s->create_request(&sstart, &send, form, format, s->search_class);
+			switch (s->search_class) {
+				case SC_PHYSICAL:
+					if (!format->offset_to_pos(sstart.offset, start)
+					|| !format->offset_to_pos(send.offset, end)) {
+						throw new ht_io_exception("Internal error: can't convert offset to viewer_pos");
+					}
 					break;
-				}
-				case SC_VISUAL: {
-					ht_visual_search_result *result = (ht_visual_search_result *)format->vsearch(request, start.address, end.address);
-					if (result) {
-						format->goto_address(result->address, format);
-						delete result;
-					} else infobox("not found");
+				case SC_VISUAL:
+					*start = sstart.pos;
+					*end = send.pos;
 					break;
-				}
 			}
 		} catch (ht_exception *e) {
 			errorbox("error: %s", e->what());
 		}
-		if (request) delete request;
 	}
 	dialog->done();
 	delete dialog;
+	return result;
 }
 
-
-ht_replace_method replace_methods[] =
+static ht_replace_method replace_methods[] =
 {
 	{ "bin: hex/ascii", 0, create_form_replace_hexascii,
 		create_replace_hexascii_context, replace_bin_process },
 	{ NULL }
 };
 
-void replace_dialog(ht_format_viewer *format, UINT searchmodes)
+UINT replace_dialog(ht_format_viewer *format, UINT searchmodes, bool *cancelled)
 {
+	*cancelled = false;
 	bounds b;
 	b.w = 50;
 	b.h = 22;
@@ -1080,7 +1092,8 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 	dialog->select_search_mode(lastsearchmodeid);
 	dialog->select_replace_mode(lastreplacemodeid);
 
-	if (dialog->run(0)) {
+	UINT repl_count = 0;
+	if (dialog->run(false)) {
 		int smodeid = dialog->get_search_modeid();
 		int rmodeid = dialog->get_replace_modeid();
 		lastsearchmodeid = smodeid;
@@ -1098,7 +1111,7 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 		try {
 /* create history entry */
 			if (s->create_desc) {
-				char hist_desc[1024];	/* FIXME: possible buffer overflow */
+				char hist_desc[1024];
 				s->create_desc(hist_desc, sizeof hist_desc, sform);
 				insert_history_entry((ht_list*)find_atom(s->histid), hist_desc, sform);
 			}
@@ -1113,11 +1126,10 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 		}
 		
 		if (request) {
-			UINT n = 0;
 			FILEOFS so = start.offset, eo = end.offset;
 			ht_physical_search_result *result;
 
-			format->push_vs_history(format);
+			format->vstate_save();
 				
 			try {
 				bool replace_all = false;
@@ -1126,9 +1138,9 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 					bool do_replace = false;
 
 					if (!replace_all) {
-						format->show_search_result(result);
-						int r = msgbox(msgbox_yes+msgbox_no+msgbox_all+msgbox_cancel, "confirmation", 0, align_center, "replace ?");
-						format->pop_vs_history();
+						bool showed = format->show_search_result(result);
+						int r = msgbox(btmask_yes+btmask_no+btmask_all+btmask_cancel, "confirmation", 0, align_center, "replace ?");
+						if (showed) app->sendmsg(cmd_vstate_restore);
 						switch (r) {
 							case button_yes:
 								do_replace = true;
@@ -1140,6 +1152,7 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 								break;
 						}
 						if (r == button_cancel) {
+							*cancelled = true;
 							delete result;
 							break;
 						}
@@ -1158,7 +1171,7 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 							format->sendmsg(msg_filesize_changed);
 						}
 						so = result->offset + irepllen;
-						n++;
+						repl_count++;
 					} else {
 						so = result->offset + result->size;
 					}
@@ -1167,15 +1180,15 @@ void replace_dialog(ht_format_viewer *format, UINT searchmodes)
 			} catch (ht_exception *e) {
 				errorbox("error: %s", e->what());
 			}
-		
-			format->pop_vs_history();
-			infobox("%d replacement(s) made", n);
+
+			app->sendmsg(cmd_vstate_restore);
 		}
 				
 		if (request) delete request;
 	}
 	dialog->done();
 	delete dialog;
+	return repl_count;
 }
 
 #define REPLACE_COPY_BUF_SIZE	64*1024
@@ -1202,9 +1215,7 @@ ht_data* create_replace_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len,
 
 bool replace_bin_process(ht_data *context, ht_text *progress_indicator)
 {
-	char status[128];
-	sprintf(status, "replacing...\n");
-	progress_indicator->settext(status);
+	progress_indicator->settext("replacing...\n");
 	
 	ht_replace_bin_context *c = (ht_replace_bin_context*)context;
 	if (c->repllen > c->len) {
@@ -1223,30 +1234,36 @@ bool replace_bin_process(ht_data *context, ht_text *progress_indicator)
 			c->o = c->ofs + c->len;
 		}
 		c->file->seek(c->o);
-		c->file->read(c->buf, c->z);
+		if (c->file->read(c->buf, c->z) != c->z)
+			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		c->file->seek(c->o+c->repllen-c->len);
-		c->file->write(c->buf, c->z);
+		if (c->file->write(c->buf, c->z) != c->z)
+			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 			
 		if (c->o > c->ofs + c->len) return true;
 		
 		c->file->seek(c->ofs);
-		c->file->write(c->repl, c->repllen);
+		if (c->file->write(c->repl, c->repllen) != c->repllen)
+				throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		free(c->buf);
 	} else if (c->repllen < c->len) {
 /* shrink */
 		UINT size = c->file->get_size();
 		if (c->o == c->ofs + c->len) {
 			c->file->seek(c->ofs);
-			c->file->write(c->repl, c->repllen);
+			if (c->file->write(c->repl, c->repllen) != c->repllen)
+				throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		}
 		
 		if (c->z > size - c->o) {
 			c->z = size - c->o;
 		}
 		c->file->seek(c->o);
-		c->file->read(c->buf, c->z);
+		if (c->file->read(c->buf, c->z) != c->z)
+			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		c->file->seek(c->o - (c->len - c->repllen));
-		c->file->write(c->buf, c->z);
+		if (c->file->write(c->buf, c->z) != c->z)
+			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		c->o += REPLACE_COPY_BUF_SIZE;
 		
 		if (c->z == REPLACE_COPY_BUF_SIZE) return true;
@@ -1255,7 +1272,8 @@ bool replace_bin_process(ht_data *context, ht_text *progress_indicator)
 		free(c->buf);
 	} else {
 		c->file->seek(c->ofs);
-		c->file->write(c->repl, c->repllen);
+		if (c->file->write(c->repl, c->repllen) != c->repllen)
+				throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 	}
 	if (c->return_repllen) *c->return_repllen = c->repllen;
 	return false;
@@ -1265,12 +1283,13 @@ bool replace_bin_process(ht_data *context, ht_text *progress_indicator)
  *	CLASS ht_search_dialog
  */
 
-void ht_search_dialog::init(bounds *b)
+void ht_search_dialog::init(bounds *b, const char *title)
 {
-	ht_dialog::init(b, "search", FS_KILLER | FS_TITLE | FS_MOVE);
+	ht_dialog::init(b, title, FS_KILLER | FS_TITLE | FS_MOVE);
 	VIEW_DEBUG_NAME("ht_search_dialog");
 
-	smodecount=0;
+	smodecount = 0;
+	smodeidx = -1;
 
 	bounds c;
 	c.x=1;
@@ -1319,9 +1338,9 @@ void ht_search_dialog::handlemsg(htmsg *msg)
 	if (msg->msg==msg_keypressed) {
 		ht_dialog::handlemsg(msg);
 		ht_listpopup_data data;
-		search_mode_popup->databuf_get(&data);
-		if ((int)data.cursor_id!=smodeidx) {
-			smodeidx=data.cursor_id;
+		search_mode_popup->databuf_get(&data, sizeof data);
+		if ((int)data.cursor_pos != smodeidx) {
+			smodeidx = data.cursor_pos;
 			select_search_mode_bymodeidx();
 		}
 	} else ht_dialog::handlemsg(msg);
@@ -1361,8 +1380,8 @@ void ht_search_dialog::select_search_mode(int id)
 void ht_search_dialog::select_search_mode_bymodeidx()
 {
 	ht_listpopup_data d;
-	d.cursor_id=smodeidx;
-	search_mode_popup->databuf_set(&d);
+	d.cursor_pos = smodeidx;
+	search_mode_popup->databuf_set(&d, sizeof d);
 	focus(smodes[smodeidx].view);
 	sendmsg(msg_dirtyview, 0);
 }
@@ -1373,7 +1392,7 @@ void ht_search_dialog::select_search_mode_bymodeidx()
 
 void ht_replace_dialog::init(bounds *b)
 {
-	ht_search_dialog::init(b);
+	ht_search_dialog::init(b, "replace");
 
 	rmodecount=0;
 
@@ -1429,12 +1448,12 @@ int ht_replace_dialog::find_replace_mode(int id)
 
 void ht_replace_dialog::handlemsg(htmsg *msg)
 {
-	if (msg->msg==msg_keypressed) {
+	if (msg->msg == msg_keypressed) {
 		ht_search_dialog::handlemsg(msg);
 		ht_listpopup_data data;
-		replace_mode_popup->databuf_get(&data);
-		if ((int)data.cursor_id!=rmodeidx) {
-			rmodeidx=data.cursor_id;
+		replace_mode_popup->databuf_get(&data, sizeof data);
+		if ((int)data.cursor_pos != rmodeidx) {
+			rmodeidx=data.cursor_pos;
 			select_replace_mode_bymodeidx();
 		}
 	} else ht_search_dialog::handlemsg(msg);
@@ -1474,9 +1493,39 @@ void ht_replace_dialog::select_replace_mode(int id)
 void ht_replace_dialog::select_replace_mode_bymodeidx()
 {
 	ht_listpopup_data d;
-	d.cursor_id=rmodeidx;
-	replace_mode_popup->databuf_set(&d);
+	d.cursor_pos = rmodeidx;
+	replace_mode_popup->databuf_set(&d, sizeof d);
 //	focus(rmodes[rmodeidx].view);
 	sendmsg(msg_dirtyview, 0);
 }
 
+/*
+ *
+ */
+ 
+ht_search_result *linear_bin_search(ht_search_request *search, FILEOFS start, FILEOFS end, ht_streamfile *file, FILEOFS fofs, dword fsize)
+{
+	ht_fxbin_search_request *s=(ht_fxbin_search_request*)search;
+		
+	int fl=(search->flags & SFBIN_CASEINSENSITIVE) ? SFBIN_CASEINSENSITIVE : 0;
+	if (start<fofs) start=fofs;
+	if (end>fofs+fsize) end=fofs+fsize;
+	if ((fsize) && (start<end)) {
+		/* create result */
+		bool search_success = false;
+		FILEOFS search_ofs;
+		ht_data *ctx = create_search_bin_context(file, start, end-start, s->data, s->data_size, fl, &search_ofs, &search_success);
+		if (execute_process(search_bin_process, ctx)) {
+			delete ctx;
+			if (search_success) {
+				ht_physical_search_result *r=new ht_physical_search_result();
+				r->offset = search_ofs;
+				r->size = s->data_size;
+				return r;
+			}
+		} else delete ctx;
+	}
+	return NULL;
+}
+
+ 
