@@ -2,7 +2,7 @@
  *	HT Editor
  *	textfile.cc
  *
- *	Copyright (C) 2001 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -37,57 +37,6 @@
  *	CLASS ht_textfile
  */
 
-bool ht_textfile::convert_ofs2line(FILEOFS o, UINT *line, UINT *pofs)
-{
-	return false;
-}
-
-bool ht_textfile::convert_line2ofs(UINT line, UINT pofs, FILEOFS *o)
-{
-	return false;
-}
-
-void ht_textfile::delete_lines(UINT line, UINT count)
-{
-}
-
-void ht_textfile::delete_chars(UINT line, UINT ofs, UINT count)
-{
-}
-
-bool ht_textfile::get_char(UINT line, char *ch, UINT pos)
-{
-	return false;
-}
-
-bool ht_textfile::getline(UINT line, UINT pofs, void *buf, UINT buflen, UINT *retlen, lexer_state *state)
-{
-	return false;
-}
-
-UINT ht_textfile::getlinelength(UINT line)
-{
-	return 0;
-}
-
-void ht_textfile::insert_lines(UINT before, UINT count)
-{
-}
-
-void ht_textfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
-{
-}
-
-bool ht_textfile::has_line(UINT line)
-{
-	return false;
-}
-
-UINT ht_textfile::linecount()
-{
-	return 0;
-}
-
 /*
  *	CLASS ht_layer_textfile
  */
@@ -132,9 +81,9 @@ UINT ht_layer_textfile::getlinelength(UINT line)
 	return ((ht_textfile*)streamfile)->getlinelength(line);
 }
 
-void ht_layer_textfile::insert_lines(UINT before, UINT count)
+void ht_layer_textfile::insert_lines(UINT before, UINT count, void **line_ends, int *line_end_lens)
 {
-	((ht_textfile*)streamfile)->insert_lines(before, count);
+	((ht_textfile*)streamfile)->insert_lines(before, count, line_ends, line_end_lens);
 }
 
 void ht_layer_textfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
@@ -152,6 +101,19 @@ UINT ht_layer_textfile::linecount()
 	return ((ht_textfile*)streamfile)->linecount();
 }
 
+void ht_layer_textfile::set_layered_assume(ht_streamfile *s, bool changes_applied)
+{
+/*	ht_streamfile *q = streamfile->get_layered();
+	if (q)*/ ((ht_textfile*)streamfile)->set_layered_assume(s, changes_applied);
+/*     else
+		streamfile = s;*/
+}
+
+void ht_layer_textfile::set_lexer(ht_syntax_lexer *lexer)
+{
+	((ht_textfile*)streamfile)->set_lexer(lexer);
+}
+
 /*
  *	CLASS ht_ltextfile_line
  */
@@ -168,40 +130,40 @@ ht_ltextfile_line::~ht_ltextfile_line()
 void	ht_ltextfile::init(ht_streamfile *streamfile, bool own_streamfile, ht_syntax_lexer *l)
 {
 	ht_textfile::init(streamfile, own_streamfile);
-	if (!get_error()) {
-		lexer=l;
-		lines=new ht_clist();
-		lines->init();
-		reread();
-		first_parse_dirty_line=linecount();
-		first_nofs_dirty_line=linecount();
-		dirty=false;
-	}
+	int e = get_error();
+	if (get_error()) return;
+	lexer = l;
+	lines = new ht_clist();
+	lines->init();
+	orig_lines = new ht_clist();
+	orig_lines->init();
+	reread();
+	first_parse_dirty_line = linecount();
+	first_nofs_dirty_line = linecount();
+	dirty = false;
+	ofs = 0;
 }
 
 void ht_ltextfile::done()
 {
 	lines->destroy();
 	delete lines;
+	orig_lines->destroy();
+	delete orig_lines;
 	ht_textfile::done();
 }
 
 void ht_ltextfile::cache_invd()
 {
-/*	UINT c=linecount();
+	lines->empty();
+	UINT c=orig_lines->count();
 	for (UINT i=0; i<c; i++) {
-		ht_ltextfile_line *e=fetch_line(i);
-		if (e->is_in_memory) {
-			free(e->in_memory.data);
-			e->is_in_memory=false;
-		}
-		if (e->on_disk.ofs==0xffffffff) {
-			lines->del(i);
-			i--;
-			c--;
-		}
-	}*/
-	reread();
+		ht_ltextfile_line *l = new ht_ltextfile_line();
+		*l = *((ht_ltextfile_line*)orig_lines->get(i));
+		lines->append(l);
+	}
+	dirty_parse(0);
+/*	reread();*/
 	dirty = false;
 }
 
@@ -255,7 +217,7 @@ bool ht_ltextfile::convert_ofs2line(FILEOFS o, UINT *line, UINT *pofs)
 
 void	ht_ltextfile::copy_to(ht_stream *stream)
 {
-	UINT c=linecount();
+/*	UINT c=linecount();
 	char buf[TEXTFILE_MAX_LINELEN+1];
 	for (UINT i=0; i<c; i++) {
 		ht_ltextfile_line *e=fetch_line(i);
@@ -265,7 +227,16 @@ void	ht_ltextfile::copy_to(ht_stream *stream)
 			stream->write(buf, s);
 			stream->write(e->lineend, e->lineendlen);
 		}
-	}
+	}*/
+#define STREAM_COPYBUF_SIZE (64*1024)
+	const UINT bufsize=STREAM_COPYBUF_SIZE;
+	byte *buf=(byte*)malloc(bufsize);
+	UINT r;
+	do {
+		r=read(buf, bufsize);
+		stream->write(buf, r);
+	} while (r);
+	free(buf);
 }
 
 void ht_ltextfile::delete_chars(UINT line, UINT ofs, UINT count)
@@ -278,8 +249,8 @@ void ht_ltextfile::delete_chars(UINT line, UINT ofs, UINT count)
 		if (ofs<olen) {
 			if (ofs+count>olen) count=olen-ofs;
 			char *nstr=(char*)malloc(olen-count);
-			memmove(nstr, ostr, ofs);
-			memmove(nstr+ofs, ostr+ofs+count, olen-ofs-count);
+			memcpy(nstr, ostr, ofs);
+			memcpy(nstr+ofs, ostr+ofs+count, olen-ofs-count);
 			free(ostr);
 			e->in_memory.data=nstr;
 			e->in_memory.len=olen-count;
@@ -330,31 +301,34 @@ UINT ht_ltextfile::find_linelen_forwd(byte *buf, UINT maxbuflen, FILEOFS ofs, in
 	UINT readlen=(maxbuflen>TEXTFILE_READSIZE) ? TEXTFILE_READSIZE : maxbuflen;
 	byte *bufp;
 	UINT s;
-	UINT len=0;
+	UINT len = 0;
 	
-	if (le_len) *le_len=0;
+	if (le_len) *le_len = 0;
 	do {
 		streamfile->seek(ofs);
-		s=streamfile->read(buf, readlen);
+		s = streamfile->read(buf, readlen);
 		int l;
-		bufp=match_lineend_forwd(buf, s, &l);
+		bufp = match_lineend_forwd(buf, s, &l);
 		if (bufp) {
-			len+=bufp-buf+l;
-			if (len>TEXTFILE_MAX_LINELEN) {
-				len=TEXTFILE_MAX_LINELEN;
-				break;
-			}
-			if (le_len) *le_len=l;
-			break;
-		} else len+=s;
-		if (len>TEXTFILE_MAX_LINELEN) {
-			len=TEXTFILE_MAX_LINELEN;
+			len += bufp-buf+l;
+			if (le_len) *le_len = l;
 			break;
 		}
-/* make sure current and next read overlap
-   to guarantee proper lineend-matching */
-		ofs+=s-(TEXTFILE_MAX_LINEENDLEN-1);
-	} while (s==readlen);
+		if (s != readlen) {
+			len += s;
+			break;
+		}
+		/* make sure current and next read overlap
+		   to guarantee proper lineend-matching */
+		if (s > (TEXTFILE_MAX_LINEENDLEN-1)) {
+			len += s-(TEXTFILE_MAX_LINEENDLEN-1);
+		}
+		ofs += s-(TEXTFILE_MAX_LINEENDLEN-1);
+	} while (s == readlen);
+	if (len > TEXTFILE_MAX_LINELEN) {
+		len = TEXTFILE_MAX_LINELEN;
+		if (le_len) *le_len = 0;
+	}
 	return len;
 }
 
@@ -377,8 +351,10 @@ ht_ltextfile_line *ht_ltextfile::fetch_line_into_memory(UINT line)
 
 UINT	ht_ltextfile::get_size()
 {
-	/* FIXME: nyi */
-	return 0;
+	int line = linecount()-1;
+	FILEOFS o = 0;
+	convert_line2ofs(line, getlinelength(line), &o);
+	return o;
 }
 
 bool ht_ltextfile::get_char(UINT line, char *ch, UINT pos)
@@ -401,6 +377,7 @@ bool ht_ltextfile::get_char(UINT line, char *ch, UINT pos)
 	return false;
 }
 
+
 bool ht_ltextfile::getline(UINT line, UINT pofs, void *buf, UINT buflen, UINT *retlen, lexer_state *state)
 {
 /* <debug> */
@@ -465,25 +442,31 @@ UINT ht_ltextfile::getlinelength_i(ht_ltextfile_line *e)
 	return 0;
 }
 
-void ht_ltextfile::insert_lines(UINT before, UINT count)
+void ht_ltextfile::insert_lines(UINT before, UINT count, void **line_ends, int *line_end_lens)
 {
-	ht_ltextfile_line *e;
+	ht_ltextfile_line *e = fetch_line(before);
+	int instate = e->instate;
 	while (count--) {
-		e=new ht_ltextfile_line();
-		e->is_in_memory=true;
-		e->instate=0;
-		e->on_disk.ofs=0xffffffff;
-		e->on_disk.len=0;
-		e->in_memory.data=(char*)malloc(1);
-		e->in_memory.len=0;
-		e->nofs=0;
-		e->lineendlen=1;
-		e->lineend[0]='\n';
+		e = new ht_ltextfile_line();
+		e->is_in_memory = true;
+		e->instate = instate;
+		e->on_disk.ofs = 0xffffffff;
+		e->on_disk.len = 0;
+		e->in_memory.data = (char*)malloc(1);
+		e->in_memory.len = 0;
+		e->nofs = 0;
+		if (line_ends && line_end_lens) {
+			e->lineendlen = *line_end_lens++;
+			memmove(e->lineend, *line_ends++, e->lineendlen);
+		} else {
+			e->lineendlen = 1;
+			e->lineend[0] = '\n';
+		}
 		lines->insert_before(e, before);
 	}
 	dirty_parse(before);
 	dirty_nofs(before);
-	dirty=true;
+	dirty = true;
 }
 
 void ht_ltextfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
@@ -504,9 +487,9 @@ void ht_ltextfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
 			nlen+=clen;
 			nstr=(char*)malloc(nlen);
 
-			memmove(nstr, ostr, ofs);
-			memmove(nstr+ofs, chars, clen);
-			memmove(nstr+ofs+clen, ostr+ofs, olen-ofs);
+			memcpy(nstr, ostr, ofs);
+			memcpy(nstr+ofs, chars, clen);
+			memcpy(nstr+ofs+clen, ostr+ofs, olen-ofs);
 
 			e->in_memory.data=nstr;
 			e->in_memory.len=nlen;
@@ -542,16 +525,12 @@ byte *ht_ltextfile::match_lineend_forwd(byte *buf, UINT buflen, int *le_len)
 {
 	byte *result=NULL;
 	
-	byte *r=(byte*)memchr(buf, '\r', buflen);
-	if (r && (r+1<buf+buflen) && (r[1]=='\n')) {
-		if ((result && (r<result)) || !result) {
-			*le_len=2;
-			result=r;
-		}
-	}
 	byte *n=(byte*)memchr(buf, '\n', buflen);
 	if (n) {
-		if ((result && (n<result)) || !result) {
+		if ((n>buf) && (n[-1] == '\r')) {
+			*le_len=2;
+			result=n-1;
+		} else {
 			*le_len=1;
 			result=n;
 		}
@@ -562,23 +541,28 @@ byte *ht_ltextfile::match_lineend_forwd(byte *buf, UINT buflen, int *le_len)
 lexer_state ht_ltextfile::next_instate(UINT line)
 {
 	byte buf[TEXTFILE_MAX_LINELEN+1];
-	lexer_state state=0;
+	lexer_state state = 0;
 
 	UINT buflen;
 	if (!getline(line, 0, buf, TEXTFILE_MAX_LINELEN, &buflen, &state)) return state;
-	buf[buflen]=0;
+	buf[buflen] = 0;
 	
 	if (lexer) {
 		text_pos p;
-		char *bufp=(char*)buf;
+		char *bufp = (char*)buf;
 		UINT toklen;
-		bool start_of_line=true;
-		p.line=line;
-		p.pofs=0;
-		while (lexer->gettoken(bufp, &state, &p, &toklen, start_of_line, true)) {
-			bufp+=toklen;
-			p.pofs+=toklen;
-			start_of_line=false;
+		bool start_of_line = true;
+		p.line = line;
+		p.pofs = 0;
+		int bufplen = buflen;
+		int prev_bufplen = -1;
+		while ((lexer->gettoken(bufp, bufplen, p, start_of_line, &state, &toklen))) {
+			bufp += toklen;
+			p.pofs += toklen;
+			bufplen -= toklen;
+			start_of_line = false;
+			if (!bufplen && !prev_bufplen) break;
+			prev_bufplen = bufplen;
 		}
 	}
 	return state;
@@ -610,52 +594,144 @@ UINT	ht_ltextfile::read(void *buf, UINT size)
 	
 	if (convert_ofs2line(o, &line, &pofs)) while (size) {
 		ht_ltextfile_line *l=fetch_line(line);
+		if (!l)
+			break;
+		UINT q;
 		if (l->is_in_memory) {
-			UINT s=l->in_memory.len;
+			q=l->in_memory.len;
+			UINT s=q;
 			if (s>pofs) {
 				s-=pofs;
 				s=MIN(size, s);
-				memmove(b+c, l->in_memory.data+pofs, s);
+				memcpy(b+c, l->in_memory.data+pofs, s);
 				size-=s;
 				c+=s;
-			} else pofs-=s;
+				pofs+=s;
+			}
 		} else {
+			q=l->on_disk.len;
 			UINT r;
-			UINT s=l->on_disk.len;
+			UINT s=q;
 			if (s>pofs) {
 				s-=pofs;
 				s=MIN(size, s);
 				streamfile->seek(l->on_disk.ofs+pofs);
 				r=streamfile->read(b+c, s);
-				size-=r;
-				c+=r;
 				if (r!=s) break;
-			} else pofs-=s;
+				size-=s;
+				c+=s;
+				pofs+=s;
+			}
 		}
 		UINT s=l->lineendlen;
-		s=MIN(size, s);
-		memmove(b+c, l->lineend, s);
-		size-=s;
-		c+=s;
+		if ((q+s>pofs) && (pofs >= q)) {
+			s-=pofs-q;
+			s=MIN(size, s);
+			memcpy(b+c, l->lineend+(pofs-q), s);
+			size-=s;
+			c+=s;
+		}
 		line++;
 		pofs=0;
 	}
+	ofs += c;
 	return c;
 }
 
-void ht_ltextfile::split_line(UINT a, UINT pos)
+void ht_ltextfile::reread()
 {
-	UINT l=getlinelength(a);
-	if (pos>l) pos=l;
-	char *aline=(char*)malloc(pos+1);
-	UINT alinelen;
-	getline(a, 0, aline, pos+1, &alinelen, NULL);
-	
-	insert_lines(a, 1);
-	insert_chars(a, 0, aline, alinelen);
+	lines->empty();
+	orig_lines->empty();
+	dirty=false;
+	FILEOFS ofs=0;
+	int ll, pll=-1, ln=0;
+	bool firstline=true;
+	byte buf[TEXTFILE_MAX_LINELEN+1];
 
-	delete_chars(a+1, 0, pos);
-	free(aline);
+	lexer_state state=0;
+	ht_ltextfile_line *e, *ce;
+
+	if (lexer) state=lexer->getinitstate();
+
+	UINT l=0;
+	while ((l=find_linelen_forwd(buf, sizeof buf, ofs, &ll))) {
+		if (streamfile->seek(ofs) != 0) break;
+		UINT x=streamfile->read(buf, l-ll);
+		buf[x]=0;
+
+		e=new ht_ltextfile_line();
+		e->is_in_memory=false;
+		e->instate=state;
+		e->on_disk.ofs=ofs;
+		e->on_disk.len=l-ll;
+		e->nofs=ofs;
+		e->lineendlen=ll;
+		assert( (UINT)ll <= sizeof e->lineend);
+		streamfile->read(e->lineend, ll);
+		lines->append(e);
+		ce=new ht_ltextfile_line();
+		*ce = *e;
+		orig_lines->append(ce);
+
+		if (lexer) {
+			text_pos p;
+			char *bufp=(char*)buf;
+			UINT toklen;
+			bool start_of_line=true;
+			p.line=ln;
+			p.pofs=0;
+			while (lexer->gettoken(bufp, l-ll-(bufp-(char*)buf), p, start_of_line, &state, &toklen)) {
+				bufp+=toklen;
+				p.pofs+=toklen;
+				start_of_line=false;
+			}
+		}
+
+		ofs+=l;
+		pll=ll;
+		firstline=false;
+		ln++;
+	}
+
+	if (pll || firstline) {
+		ll=0;
+		e=new ht_ltextfile_line();
+		e->is_in_memory=false;
+		e->instate=state;
+		e->on_disk.ofs=ofs;
+		e->on_disk.len=l-ll;
+		e->nofs=ofs;
+		e->lineendlen=ll;
+		lines->append(e);
+		ce=new ht_ltextfile_line();
+		*ce = *e;
+		orig_lines->append(ce);
+	}
+}
+
+void ht_ltextfile::split_line(UINT a, UINT pos, void *line_end, int line_end_len)
+{
+	if (pos == 0) {
+		insert_lines(a, 1, &line_end, &line_end_len);
+	} else {
+		UINT l=getlinelength(a);
+		if (pos>l) pos=l;
+		char *aline=(char*)malloc(pos+1);
+		UINT alinelen;
+		getline(a, 0, aline, pos+1, &alinelen, NULL);
+	
+		insert_lines(a, 1, &line_end, &line_end_len);
+		insert_chars(a, 0, aline, alinelen);
+
+		delete_chars(a+1, 0, pos);
+		free(aline);
+	}
+}
+
+int	ht_ltextfile::seek(FILEOFS offset)
+{
+	ofs = offset;
+	return 0;
 }
 
 void ht_ltextfile::set_layered(ht_streamfile *streamfile)
@@ -664,12 +740,98 @@ void ht_ltextfile::set_layered(ht_streamfile *streamfile)
 	reread();
 }
 
+void ht_ltextfile::set_layered_assume(ht_streamfile *streamfile, bool changes_applied)
+{
+	if (changes_applied) {
+		orig_lines->empty();
+		UINT c = lines->count();
+		for (UINT i=0; i<c; i++) {
+			ht_ltextfile_line *l = fetch_line_nofs_ok(i);
+			ht_ltextfile_line *m = new ht_ltextfile_line();
+			l->on_disk.ofs = l->nofs;
+			if (l->is_in_memory) {
+				l->on_disk.len = l->in_memory.len;
+				free(l->in_memory.data);
+				l->is_in_memory = false;
+			}
+			*m = *l;
+			orig_lines->append(m);
+		}
+		cache_invd();
+	}
+	ht_layer_streamfile::set_layered(streamfile);
+}
+
+void ht_ltextfile::set_lexer(ht_syntax_lexer *l)
+{
+	if (lexer != l) {
+		lexer = l;
+		dirty_parse(0);
+	}
+}
+
+FILEOFS ht_ltextfile::tell()
+{
+	return ofs;
+}
+
 int	ht_ltextfile::truncate(UINT newsize)
 {
 	/* FIXME: nyi */
 	return ENOSYS;
 }
 
+void ht_ltextfile::update_parse(UINT target)
+{
+	ht_ltextfile_line *e;
+	UINT line = first_parse_dirty_line;
+
+	lexer_state instate=0;
+	if (line) {
+		instate = next_instate(line-1);
+// FIXME: function help works with this already...
+/*		e = fetch_line(line);
+		instate = e->instate;*/
+	} else {
+		if (lexer) instate = lexer->getinitstate();
+	}
+
+	while (line<=target) {
+		e = fetch_line(line);
+		if (!e) break;
+//		if (e->instate==instate) break;	/* FIXME: valid simplification ?!... */
+		e->instate = instate;
+		first_parse_dirty_line = line+1;
+		instate = next_instate(line);
+		line++;
+	}
+	e = fetch_line(line);
+	if (e) e->instate = instate;
+}
+
+void ht_ltextfile::update_nofs(UINT target)
+{
+	ht_ltextfile_line *e;
+	UINT line=first_nofs_dirty_line;
+
+	FILEOFS nofs;
+	if (line) {
+		e=fetch_line(line-1);
+		nofs=next_nofs(e);
+	} else {
+		nofs=0;
+	}
+
+	while (line<=target) {
+		e=fetch_line(line);
+		if (!e) break;
+		e->nofs=nofs;
+		first_nofs_dirty_line=line+1;
+		nofs=next_nofs(e);
+		line++;
+	}
+}
+
 int ht_ltextfile::vcntl(UINT cmd, va_list vargs)
 {
 	switch (cmd) {
@@ -684,152 +846,46 @@ int ht_ltextfile::vcntl(UINT cmd, va_list vargs)
 				UINT s=va_arg(vargs, UINT);
 				bool *b=va_arg(vargs, bool*);
 				*b = dirty;
+				o = 0;	// gcc warns otherwise
+				s = 0;    // gcc warns otherwise
 				return 0;
 		}
 		case FCNTL_MODS_CLEAR_DIRTY: {
-				dirty = false;
-				return 0;
+			dirty = false;
+			return 0;
 		}
 	}
 	return ht_textfile::vcntl(cmd, vargs);
 }
 
-UINT	ht_ltextfile::write(void *buf, UINT size)
+UINT	ht_ltextfile::write(const void *buf, UINT size)
 {
-	FILEOFS o=tell();
+	FILEOFS o = tell();
 	UINT line;
 	UINT pofs;
-	byte *b=(byte*)buf;
+	byte *b = (byte*)buf;
+	UINT r = 0;
 	if (convert_ofs2line(o, &line, &pofs)) {
+		r = size;
 		while (size) {
 			int lelen;
 			UINT s;
-			byte *c=match_lineend_forwd(b, size, &lelen);
+			byte *c = match_lineend_forwd(b, size, &lelen);
 			if (c) {
-				s=c-b;
-				split_line(line, pofs);
+				s = c-b;
+				split_line(line, pofs, c, lelen);
 			} else {
-				s=size;
-				lelen=0;
+				s = size;
+				lelen = 0;
 			}
 			insert_chars(line, pofs, b, s);
-			s+=lelen;
-			b+=s;
-			size-=s;
+			s += lelen;
+			b += s;
+			size -= s;
 			line++;
-			pofs=0;
+			pofs = 0;
 		}
 	}
-	return 0;
-}
-
-void ht_ltextfile::reread()
-{
-	lines->empty();
-	dirty=false;
-	FILEOFS ofs=0;
-	int ll, pll=-1, ln=0;
-	bool firstline=true;
-	byte buf[TEXTFILE_MAX_LINELEN+1];
-
-	lexer_state state=0;
-	ht_ltextfile_line *e;
-
-	if (lexer) state=lexer->getinitstate();
-	
-	UINT l=0;
-	while ((l=find_linelen_forwd(buf, sizeof buf, ofs, &ll))) {
-		streamfile->seek(ofs);
-		UINT x=streamfile->read(buf, l-ll);
-		buf[x]=0;
-
-		e=new ht_ltextfile_line();
-		e->is_in_memory=false;
-		e->instate=state;
-		e->on_disk.ofs=ofs;
-		e->on_disk.len=l-ll;
-		e->nofs=ofs;
-		e->lineendlen=ll;
-		assert( (UINT)ll <=sizeof e->lineend);
-		streamfile->read(e->lineend, ll);
-		lines->append(e);
-
-		if (lexer) {
-			text_pos p;
-			char *bufp=(char*)buf;
-			UINT toklen;
-			bool start_of_line=true;
-			p.line=ln;
-			p.pofs=0;
-			while (lexer->gettoken(bufp, &state, &p, &toklen, start_of_line, true)) {
-				bufp+=toklen;
-				p.pofs+=toklen;
-				start_of_line=false;
-			}
-		}
-	
-		ofs+=l;
-		pll=ll;
-		firstline=false;
-		ln++;
-	}
-
-	if (pll || firstline) {
-		ll=0;
-		e=new ht_ltextfile_line();
-		e->is_in_memory=false;
-		e->instate=state;
-		e->on_disk.ofs=ofs;
-		e->on_disk.len=l-ll;
-		e->nofs=ofs;
-		e->lineendlen=ll;
-
-		lines->append(e);
-	}
-}
-
-void ht_ltextfile::update_parse(UINT target)
-{
-	ht_ltextfile_line *e;
-	UINT line=first_parse_dirty_line;
-
-	lexer_state instate=0;
-	if (line) {
-		instate=next_instate(line-1);
-	} else {
-		if (lexer) instate=lexer->getinitstate();
-	}
-	
-	while (line<=target) {
-		e=fetch_line(line);
-		if (!e) break;
-//		if (e->instate==instate) break;	/* FIXME: valid simplification ?!... */
-		e->instate=instate;
-		first_parse_dirty_line=line+1;
-		instate=next_instate(line);
-		line++;
-	}
-}
-
-void ht_ltextfile::update_nofs(UINT target)
-{
-	ht_ltextfile_line *e;
-	UINT line=first_nofs_dirty_line;
-
-	FILEOFS nofs;
-	if (line) {
-		e=fetch_line(line-1);
-		nofs=next_nofs(e);
-	} else {
-		nofs=0;
-	}
-	
-	while (line<=target) {
-		e=fetch_line(line);
-		if (!e) break;
-		e->nofs=nofs;
-		first_nofs_dirty_line=line+1;
-		nofs=next_nofs(e);
-		line++;
-	}
+	ofs += r;
+	return r;
 }
